/************************************************************************//**
 *
 * @file    pwg2_reg.h
 *
 * @brief   Definition of the Power Generator(PWG2) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef PWG2_REG_H__
#define PWG2_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// PWG2 Control Register.
typedef union PWGCTL_REG_TAG {
    unsigned short  reg;
    struct PWGCTL_REG_BIT_TAG {
        /**
         * These bits control the PWG2 operating mode.
         * PWGMOD[2:0]
         * These bits control the PWG2 operating mode.
         *
         * +-------------+---------------------+
         * | PWGMOD[2:0] |  Operating mode     |
         * +-------------+---------------------+
         * |  0x7-0x6    |      Reserved       |
         * +-------------+---------------------+
         * |    0x5      | Super economy mode  |
         * +-------------+---------------------+
         * |    0x4      |      Reserved       |
         * +-------------+---------------------+
         * |    0x3      |    Economy mode     |
         * +-------------+---------------------+
         * |    0x2      |     Normal mode     |
         * +-------------+---------------------+
         * |    0x1      |      Reserved       |
         * +-------------+---------------------+
         * |    0x0      |   Automatic mode    |
         * +-------------+---------------------+
         */
        unsigned short      PWGMOD      : 3;
        unsigned short      reserved_1  :13;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} PWGCTL_REG;

/// PWG2 Timing Control Register.
typedef union PWGTIM_REG_TAG {
    unsigned short  reg;
    struct PWGTIM_REG_BIT_TAG {
        /**
         * These bits set the charge pump operating clock (select an OSC1 clock division ratio).
         * DCCCLK[1:0]
         * These bits set the charge pump operating clock (select an OSC1 clock division ratio).
         *
         * +--------------+---------------------+
         * |  DCCCLK[1:0] | OSC1 division ratio |
         * +--------------+---------------------+
         * |     0x3      |        1/256        |
         * +--------------+---------------------+
         * |     0x2      |        1/128        |
         * +--------------+---------------------+
         * |     0x1      |        1/64         |
         * +--------------+---------------------+
         * |     0x0      |        1/32         |
         * +--------------+---------------------+
         */
        unsigned short      DCCCLK      : 2;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} PWGTIM_REG;

/// PWG2 Interrupt Flag Register.
typedef union PWGINTF_REG_TAG {
    unsigned short  reg;
    struct PWGINTF_REG_BIT_TAG {
        /**
         * This bit indicates the PWG2 mode transition completion interrupt cause occurrence status.
         *
         * .MODCMPIF
         * This bit indicates the PWG2 mode transition completion interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      MODCMPIF    : 1;
        unsigned short      reserved_1  :15;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} PWGINTF_REG;

/// PWG2 Interrupt Enable Register.
typedef union PWGINTE_REG_TAG {
    unsigned short  reg;
    struct PWGINTE_REG_BIT_TAG {
        /**
         * This bit enable the PWG2 mode transition completion interrupt.
         * .MODCMPIE
         * This bit enable the PWG2 mode transition completion interrupt.
         *    1 (R/W): Enable interrupt
         *    0 (R/W): Disable interrupt
         */
        unsigned short      MODCMPIE    : 1;
        unsigned short      reserved_1  :15;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} PWGINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // PWG2_REG_H__
