/************************************************************************//**
 *
 * @file    wdt2_reg.h
 *
 * @brief   Definition of the Watchdog Timer(WDT2) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef WDT2_REG_H__
#define WDT2_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <c17_mcu_select.h>

/// WDT2 Clock Control Register
typedef union WDTCLK_REG_TAG {
    unsigned short  reg;
    struct WDTCLK_REG_BIT_TAG {
#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
    defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
        /**
         * These bits select the clock source of WDT2.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/16,384|1/128   |1/16,384|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/8,192 |        |1/8,192 |        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/4,096 |        |1/4,096 |        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/2,048 |        |1/2,048 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#elif defined(C17_MCUSEL_M40) || \
    defined(C17_MCUSEL_F63)
        /**
         * These bits select the clock source of WDT2.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/16,384|1/128   |1/65,536|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/8,192 |        |1/32,768|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/4,096 |        |1/16,384|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/2,048 |        |1/8,192 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#else
        /**
         * These bits select the clock source of WDT2.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/65,536|1/128   |1/65,536|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/32,768|        |1/32,768|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/16,384|        |1/16,384|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/8,192 |        |1/8,192 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#endif
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
    defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
        /**
         * These bits select the division ratio of the WDT2 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/16,384|1/128   |1/16,384|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/8,192 |        |1/8,192 |        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/4,096 |        |1/4,096 |        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/2,048 |        |1/2,048 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#elif defined(C17_MCUSEL_M40) || \
    defined(C17_MCUSEL_F63)
        /**
         * These bits select the division ratio of the WDT2 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/16,384|1/128   |1/65,536|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/8,192 |        |1/32,768|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/4,096 |        |1/16,384|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/2,048 |        |1/8,192 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#else
        /**
         * These bits select the division ratio of the WDT2 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of WDT2.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the WDT2 operating clock (counter clock). The clock frequency.
         *  should be set to around 256 Hz.
         *  +-------------+-----------------------------------+
         *  |             |           CLKSRC[1:0]             |
         *  |             +--------+--------+--------+--------+
         *  |             |  0x0   |   0x1  |  0x2   |  0x3   |
         *  | CLKDIV[1:0] +--------+--------+--------+--------+
         *  |             |        |        |        |        |
         *  |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         *  |             |        |        |        |        |
         *  +-------------+--------+--------+--------+--------+
         *  |     0x3     |1/65,536|1/128   |1/65,536|1/1     |
         *  +-------------+--------+        +--------+        +
         *  |     0x2     |1/32,768|        |1/32,768|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x1     |1/16,384|        |1/16,384|        |
         *  +-------------+--------+        +--------+        +
         *  |     0x0     |1/8,192 |        |1/8,192 |        |
         *  +-------------+--------+--------+--------+--------+
         */
#endif
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the WDT2 operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the WDT2 operating clock is supplied in DEBUG mode or not.
         *    1 (R/WP): Clock supplied in DEBUG mode
         *    0 (R/WP): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} WDTCLK_REG;

/// WDT2 Control Register
typedef union WDTCTL_REG_TAG {
    unsigned short  reg;
    struct WDTCTL_REG_BIT_TAG {
        /**
         * These bits control WDT2 to run and stop.
         *  .WDTRUN[3:0]
         *  These bits control WDT2 to run and stop.
         *    0xa (R/WP): Stop
         *    Values other than 0xa (R/WP): Run
         */
        unsigned short      WDTRUN      : 4;
        /**
         * This bit resets WDT2.
         * .WDTCNTRST
         *  This bit resets the 10-bit counter and the WDTCTL.STATNMI bit.
         *    1 (WP): Reset
         *    0 (WP): Ignored
         *    0 (R): Always 0 when being read
         */
        unsigned short      WDTCNTRST   : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit indicates that a counter overflow and NMI have occurred.
         * .STATNMI
         *  This bit indicates that a counter compare match and NMI have occurred.
         *    1 (R): NMI (counter compare match) occurred
         *    0 (R): NMI not occurred
         */
        unsigned short      STATNMI     : 1;
        /**
         * These bits set the WDT2 operating mode.
         * .MOD[1:0]
         *  These bits set the WDT2 operating mode.
         * +----------+----------------+------------------------------------------------------------+
         * | MOD[1:0] | Operating mode | Description                                                |
         * +----------+----------------+------------------------------------------------------------+
         * |   0x3    |  Reserved      |    -                                                       |
         * +----------+----------------+------------------------------------------------------------+
         * |   0x2    |  RESET after   | If the WDTCTL.STATNMI bit is not cleared to 0 after an NMI |
         * |          |  NMI mode      | has occurred due to a counter compare match, WDT2 issues   |
         * |          |                | a reset when the next compare match occurs.                |
         * +----------+----------------+------------------------------------------------------------+
         * |   0x1    |  NMI mode      | WDT2 issues an NMI when a counter compare match occurs.    |
         * +----------+----------------+------------------------------------------------------------+
         * |   0x0    |  RESET mode    | WDT2 issues a reset when a counter compare match occurs.   |
         * +----------+----------------+------------------------------------------------------------+
         */
        unsigned short      MOD         : 2;
        unsigned short      reserved_2  : 5;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} WDTCTL_REG;

/// WDT2 Counter Compare Match Register.
typedef union WDTCMP_REG_TAG {
    unsigned short  reg;
    struct WDTCMP_REG_BIT_TAG {
        /**
         * These bits set the NMI/reset generation cycle.
         * .CMP[9:0]
         *  These bits set the NMI/reset generation cycle.
         *  The value set in this register is compared with the 10-bit counter value while WDT2 is running, and
         *  an NMI or reset is generated when they are matched.
         */
        unsigned short      CMP         :10;
        unsigned short      reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} WDTCMP_REG;

#ifdef __cplusplus
}
#endif

#endif // WDT2_REG_H__
