/************************************************************************//**
 *
 * @file    c17_rtca2.c
 *
 * @brief   This is RTCA2 Registers setting program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_rtca2.h"
#include "c17_init_config.h"

#if defined(C17_PE_RTCA)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector08_handler, c17intRtca2);	// RTCA2 interrupt
#endif /* CRT0_LINKED */

/* --- global variable --- */
// Interrupt counter.
/**
 * @todo c17rtcaIntCount = Interruption counter.
 */
volatile unsigned int c17rtca2IntCount = 0;		///< Interruption counter.

/**
 * @todo c17swIntCount = Interruption counter.
 */
volatile unsigned int c17swIntCount = 0;		///< Interruption counter.

// Theoretical Regulation.
volatile unsigned long c17rtca2TrCount;			///< The theoretical regulation execution cycle time counter.

/* --- static --- */
static unsigned char numtobcd(unsigned char num);
static unsigned char bcdtonum(unsigned char bcd);

/************************************************************************//**
 * @brief   initRtca2.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17initRtca2(void)
{
	RTCCTLL_REG	rtcctll_set;
	RTCSEC1_REG	rtcsec_set;
	RTCHUR1_REG	rtchur_set;
	RTCMON1_REG	rtcmon_set;
	RTCYAR1_REG	rtcyar_set;

	CLGOSC_REG	clgosc_set;

	/// When OSC1 doesn't start, nothing is done.
	/* CLGOSC:CLG Oscillation Control Register. */
	{
		clgosc_set.reg = CLGOSC;
	}

	if(clgosc_set.bit.OSC1EN == 0) {
		return EXIT_FAILURE;
	}

	/// Stop RTCA2.
	c17stopRtca2();

	/// Hour counter set to 24H mode.
	/* RTCCTL: RTC Control Register. */
	{
	   /**
		* @todo change RTC24H.
		*/
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTC24H = C17_RTCA_24H;
		RTCCTLL = rtcctll_set.reg;
	}

	/// "01/01/00 Sunday 00:00:00" setting.
   /**
	* @todo change default time.(Main Time Zone)
	*/
	{
		rtcsec_set.reg = RTCSEC1;
		rtcsec_set.bit.RTCSH = 0;
		rtcsec_set.bit.RTCSL = 0;
		rtchur_set.bit.RTCAP = 0;
		rtchur_set.bit.RTCHH = 0;
		rtchur_set.bit.RTCHL = 0;
		rtchur_set.bit.RTCMIH = 0;
		rtchur_set.bit.RTCMIL = 0;
		rtcmon_set.bit.RTCMOH = 0;
		rtcmon_set.bit.RTCMOL = 1;
		rtcmon_set.bit.RTCDH = 0;
		rtcmon_set.bit.RTCDL = 1;
		rtcyar_set.bit.RTCWK = 0;
		rtcyar_set.bit.RTCYH = 0;
		rtcyar_set.bit.RTCYL = 0;
		RTCSEC1 = rtcsec_set.reg;
		RTCHUR1 = rtchur_set.reg;
		RTCMON1 = rtcmon_set.reg;
		RTCYAR1 = rtcyar_set.reg;
	}
	/// "01/01/00 Sunday 00:00:00" setting.
   /**
	* @todo change default time.(Time Zone 2)
	*/
	{
		rtcsec_set.reg = RTCSEC2;
		rtcsec_set.bit.RTCSH = 0;
		rtcsec_set.bit.RTCSL = 0;
		rtchur_set.bit.RTCAP = 0;
		rtchur_set.bit.RTCHH = 0;
		rtchur_set.bit.RTCHL = 0;
		rtchur_set.bit.RTCMIH = 0;
		rtchur_set.bit.RTCMIL = 0;
		rtcmon_set.bit.RTCMOH = 0;
		rtcmon_set.bit.RTCMOL = 1;
		rtcmon_set.bit.RTCDH = 0;
		rtcmon_set.bit.RTCDL = 1;
		rtcyar_set.bit.RTCWK = 0;
		rtcyar_set.bit.RTCYH = 0;
		rtcyar_set.bit.RTCYL = 0;
		RTCSEC2 = rtcsec_set.reg;
		RTCHUR2 = rtchur_set.reg;
		RTCMON2 = rtcmon_set.reg;
		RTCYAR2 = rtcyar_set.reg;
	}
	/// "01/01/00 Sunday 00:00:00" setting.
   /**
	* @todo change default time.(Time Zone 3)
	*/
	{
		rtcsec_set.reg = RTCSEC3;
		rtcsec_set.bit.RTCSH = 0;
		rtcsec_set.bit.RTCSL = 0;
		rtchur_set.bit.RTCAP = 0;
		rtchur_set.bit.RTCHH = 0;
		rtchur_set.bit.RTCHL = 0;
		rtchur_set.bit.RTCMIH = 0;
		rtchur_set.bit.RTCMIL = 0;
		rtcmon_set.bit.RTCMOH = 0;
		rtcmon_set.bit.RTCMOL = 1;
		rtcmon_set.bit.RTCDH = 0;
		rtcmon_set.bit.RTCDL = 1;
		rtcyar_set.bit.RTCWK = 0;
		rtcyar_set.bit.RTCYH = 0;
		rtcyar_set.bit.RTCYL = 0;
		RTCSEC3 = rtcsec_set.reg;
		RTCHUR3 = rtchur_set.reg;
		RTCMON3 = rtcmon_set.reg;
		RTCYAR3 = rtcyar_set.reg;
	}


	/// The variable initialize.
	c17rtca2IntCount = 0;
	c17swIntCount = 0;
	c17rtca2TrCount = 0;

	/// Initialize the interrupt.
	ILVRTCA_0 = INTERRUPT_LEVEL_RTCA2;			/// - Set interrupt level.

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief   start Rtca2.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startRtca2(void)
{
	RTCCTLL_REG	rtcctll_set;

	/* RTCCTLL:RTC Control Register */
	{
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCRUN = 1;
		RTCCTLL = rtcctll_set.reg;
	}
}


/************************************************************************//**
 * @brief   stop Rtca2.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopRtca2(void)
{
	RTCCTLL_REG	rtcctll_set;

	/* RTCCTLL:RTC Control Register */
	{
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCRUN = 0;
		RTCCTLL = rtcctll_set.reg;
	}
}


/************************************************************************//**
 * @brief   Rtca2 interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intRtca2(void)
{
	RTCINTF_REG	rtcintf_set;
#if THEORETICAL_ENABLE
	RTCCTLH_REG	rtcctlh_set;
#endif

	/* RTC Interrupt Flag Register. Interrupt Flag Register read & clear */
	{
		rtcintf_set.reg = RTCINTF;
		RTCINTF = rtcintf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(rtcintf_set.bit.RTCTRMIF == 1)
	{
	   /**
		* @todo add interrupt function of Theoretical regulation completion.
		*/
	}
	if(rtcintf_set.bit.SW1IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 1Hz.
		*/
		c17swIntCount++;
	}
	if(rtcintf_set.bit.SW10IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 10Hz.
		*/
	}
	if(rtcintf_set.bit.SW100IF == 1)
	{
	   /**
		* @todo add interrupt function of  Stopwatch 100Hz.
		*/
	}
	if(rtcintf_set.bit.ALARMIF == 1)
	{
	   /**
		* @todo add interrupt function of  Alarm.
		*/
	}
	if(rtcintf_set.bit._1DAYIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-day.
		*/
	}
	if(rtcintf_set.bit._1HURIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-hour.
		*/
	}
	if(rtcintf_set.bit._1MINIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-minute.
		*/
	}
	if(rtcintf_set.bit._1SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1-second.
		*/
		// Count up the interrupt counter(1-second).
		c17rtca2IntCount++;

#if THEORETICAL_ENABLE
		// Execute theoretical regulation.
		c17rtca2TrCount++;
		if (c17rtca2TrCount == THEORETICAL_CYCLE_TIME) {

			/* RTCCTLH:RTCA Control Register.*/
			{
				rtcctlh_set.bit.RTCTRM = THEORETICAL_TRIM;
				RTCCTLH = rtcctlh_set.reg;
			}
			c17rtca2TrCount = 0;
		}
#endif
	}
	if(rtcintf_set.bit._1_2SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/2-second.
		*/

	}
	if(rtcintf_set.bit._1_4SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/4-second.
		*/
	}
	if(rtcintf_set.bit._1_8SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/8-second.
		*/
	}
	if(rtcintf_set.bit._1_32SECIF == 1)
	{
	   /**
		* @todo add interrupt function of  1/32-second.
		*/
	}
}


/************************************************************************//**
 * @brief   Set Rtca2 Date & time.
 *
 * @param   rtcDate
 *
 * @param   Time zone            Set value.
 *  @li @c   C17_RTCA_ZONE_MAIN  Main Time Zone.
 *  @li @c   C17_RTCA_ZONE_2     Time Zone 2.
 *  @li @c   C17_RTCA_ZONE_3     Time Zone 3.
 *  @li @c   others              Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17setRtca2Date(ST_RTCA_BCD* rtcBcd, unsigned short zone)
{
	RTCCTLL_REG	rtcctll_set;
	RTCSEC1_REG	rtcsec_set;
	RTCHUR1_REG	rtchur_set;
	RTCMON1_REG	rtcmon_set;
	RTCYAR1_REG	rtcyar_set;

	/// Check busy.
	{
		rtcctll_set.reg = RTCCTLL;
	}

	while(rtcctll_set.bit.RTCBSY == 1) {
		// Wait...
		{
			rtcctll_set.reg = RTCCTLL;
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 1;
		RTCCTLL = rtcctll_set.reg;
	}

	/// Set value.
	{
		rtcsec_set.reg = RTCSEC1;

		rtcsec_set.bit.RTCSH = rtcBcd->second >> 4;
		rtcsec_set.bit.RTCSL = rtcBcd->second & 0x0f;
		rtchur_set.bit.RTCAP = rtcBcd->ampm;
		rtchur_set.bit.RTCHH = rtcBcd->hour >> 4;
		rtchur_set.bit.RTCHL = rtcBcd->hour & 0x0f;
		rtchur_set.bit.RTCMIH = rtcBcd->minute >> 4;
		rtchur_set.bit.RTCMIL = rtcBcd->minute & 0x0f;
		rtcmon_set.bit.RTCMOH = rtcBcd->month >> 4;
		rtcmon_set.bit.RTCMOL = rtcBcd->month & 0x0f;
		rtcmon_set.bit.RTCDH = rtcBcd->day >> 4;
		rtcmon_set.bit.RTCDL = rtcBcd->day & 0x0f;
		rtcyar_set.bit.RTCWK = rtcBcd->week;
		rtcyar_set.bit.RTCYH = rtcBcd->year >> 4;
		rtcyar_set.bit.RTCYL = rtcBcd->year & 0x0f;

		switch(zone){
		case C17_RTCA_ZONE_MAIN:
			RTCSEC1 = rtcsec_set.reg;
			RTCHUR1 = rtchur_set.reg;
			RTCMON1 = rtcmon_set.reg;
			RTCYAR1 = rtcyar_set.reg;
			break;
		case C17_RTCA_ZONE_2:
			RTCSEC2 = rtcsec_set.reg;
			RTCHUR2 = rtchur_set.reg;
			RTCMON2 = rtcmon_set.reg;
			RTCYAR2 = rtcyar_set.reg;
			break;

		case C17_RTCA_ZONE_3:
			RTCSEC3 = rtcsec_set.reg;
			RTCHUR3 = rtchur_set.reg;
			RTCMON3 = rtcmon_set.reg;
			RTCYAR3 = rtcyar_set.reg;
			break;

		default:
			break;
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 0;
		RTCCTLL = rtcctll_set.reg;
	}
}


/************************************************************************//**
 * @brief   Get Rtca2 Date & time.
 *
 * @param   rtcDate
 *
 * @param   Time zone            Set value.
 *  @li @c   C17_RTCA_ZONE_MAIN  Main Time Zone.
 *  @li @c   C17_RTCA_ZONE_2     Time Zone 2.
 *  @li @c   C17_RTCA_ZONE_3     Time Zone 3.
 *  @li @c   others              Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17getRtca2Date(ST_RTCA_BCD* rtcBcd, unsigned short zone)
{
	RTCCTLL_REG	rtcctll_set;
	RTCSEC1_REG	rtcsec_set;
	RTCHUR1_REG	rtchur_set;
	RTCMON1_REG	rtcmon_set;
	RTCYAR1_REG	rtcyar_set;

	/// Check busy.
	{
		rtcctll_set.reg = RTCCTLL;
	}

	while(rtcctll_set.bit.RTCBSY == 1) {
		// Wait...
		{
			rtcctll_set.reg = RTCCTLL;
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 1;
		RTCCTLL = rtcctll_set.reg;
	}

	/// Get value.
	{
		switch(zone){
		case C17_RTCA_ZONE_MAIN:
			rtcsec_set.reg = RTCSEC1;
			rtchur_set.reg = RTCHUR1;
			rtcmon_set.reg = RTCMON1;
			rtcyar_set.reg = RTCYAR1;
			break;
		case C17_RTCA_ZONE_2:
			rtcsec_set.reg = RTCSEC2;
			rtchur_set.reg = RTCHUR2;
			rtcmon_set.reg = RTCMON2;
			rtcyar_set.reg = RTCYAR2;
			break;

		case C17_RTCA_ZONE_3:
			rtcsec_set.reg = RTCSEC3;
			rtchur_set.reg = RTCHUR3;
			rtcmon_set.reg = RTCMON3;
			rtcyar_set.reg = RTCYAR3;
			break;

		default:
			break;
		}

		rtcBcd->second = rtcsec_set.bit.RTCSH << 4;
		rtcBcd->second |= rtcsec_set.bit.RTCSL;
		rtcBcd->ampm = rtchur_set.bit.RTCAP;
		rtcBcd->hour = rtchur_set.bit.RTCHH << 4;
		rtcBcd->hour |= rtchur_set.bit.RTCHL;
		rtcBcd->minute = rtchur_set.bit.RTCMIH  << 4;
		rtcBcd->minute |= rtchur_set.bit.RTCMIL;
		rtcBcd->month = rtcmon_set.bit.RTCMOH << 4;
		rtcBcd->month |= rtcmon_set.bit.RTCMOL;
		rtcBcd->day = rtcmon_set.bit.RTCDH << 4;
		rtcBcd->day |= rtcmon_set.bit.RTCDL;
		rtcBcd->week = rtcyar_set.bit.RTCWK;
		rtcBcd->year = rtcyar_set.bit.RTCYH << 4;
		rtcBcd->year |= rtcyar_set.bit.RTCYL;
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 0;
		RTCCTLL = rtcctll_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set Rtca2 Alarm .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17setRtca2Alarm(ST_RTCA_BCD* rtcBcd)
{
	RTCCTLL_REG	rtcctll_set;
	RTCALM1_REG	rtcarm1_set;
	RTCALM2_REG	rtcarm2_set;

	/// Check busy.
	{
		rtcctll_set.reg = RTCCTLL;
	}

	while(rtcctll_set.bit.RTCBSY == 1) {
		// Wait...
		{
			rtcctll_set.reg = RTCCTLL;
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 1;
		RTCCTLL = rtcctll_set.reg;
	}

	/// Set value.
	{
		rtcarm1_set.bit.RTCSHA = rtcBcd->second >> 4;
		rtcarm1_set.bit.RTCSLA = rtcBcd->second & 0x0f;
		rtcarm2_set.bit.RTCAPA = rtcBcd->ampm;
		rtcarm2_set.bit.RTCHHA = rtcBcd->hour >> 4;
		rtcarm2_set.bit.RTCHLA = rtcBcd->hour & 0x0f;
		rtcarm2_set.bit.RTCMIHA = rtcBcd->minute >> 4;
		rtcarm2_set.bit.RTCMILA = rtcBcd->minute & 0x0f;
		RTCALM1 = rtcarm1_set.reg;
		RTCALM2 = rtcarm2_set.reg;
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 0;
		RTCCTLL = rtcctll_set.reg;
	}
}


/************************************************************************//**
 * @brief   Enable Rtca2 interrupt .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntRtca2(void)
{
	RTCINTF_REG	rtcintf_set;
	RTCINTE_REG	rtcinte_set;

	/// Clear interrupt flag.
	/* RTC Interrupt Flag Register. Interrupt Flag Register read & clear */
	{
		rtcintf_set.reg = RTCINTF;
		rtcintf_set.bit.RTCTRMIF = 1;
		rtcintf_set.bit.ALARMIF = 1;
		rtcintf_set.bit._1DAYIF = 1;
		rtcintf_set.bit._1HURIF = 1;
		rtcintf_set.bit._1MINIF = 1;
		rtcintf_set.bit._1SECIF = 1;
		rtcintf_set.bit._1_2SECIF = 1;
		rtcintf_set.bit._1_4SECIF = 1;
		rtcintf_set.bit._1_8SECIF = 1;
		rtcintf_set.bit._1_32SECIF = 1;
		RTCINTF = rtcintf_set.reg;
	}

	/* RTCINTE:RTC Interrupt Enable Register */
	{
	    /**
	     * @todo Change Interrupt enable flag.
	     */
		rtcinte_set.reg = RTCINTE;
		rtcinte_set.bit.RTCTRMIE = 0;
		rtcinte_set.bit.ALARMIE = 0;
		rtcinte_set.bit._1DAYIE = 0;
		rtcinte_set.bit._1HURIE = 0;
		rtcinte_set.bit._1MINIE = 0;
		rtcinte_set.bit._1SECIE = 1;
		rtcinte_set.bit._1_2SECIE = 0;
		rtcinte_set.bit._1_4SECIE = 0;
		rtcinte_set.bit._1_8SECIE = 0;
		rtcinte_set.bit._1_32SECIE = 0;
		RTCINTE = rtcinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Disable Rtca2 interrupt .
 *
 * @param   rtcDate
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntRtca2(void)
{
	RTCINTE_REG	rtcinte_set;

	/// Disable all interrupt.
	/* RTCINTE:RTC Interrupt Enable Register */
	{
		rtcinte_set.reg = RTCINTE;
		rtcinte_set.bit.RTCTRMIE = 0;
		rtcinte_set.bit.SW1IE = 0;
		rtcinte_set.bit.SW10IE = 0;
		rtcinte_set.bit.SW100IE = 0;
		rtcinte_set.bit.ALARMIE = 0;
		rtcinte_set.bit._1DAYIE = 0;
		rtcinte_set.bit._1HURIE = 0;
		rtcinte_set.bit._1MINIE = 0;
		rtcinte_set.bit._1SECIE = 0;
		rtcinte_set.bit._1_2SECIE = 0;
		rtcinte_set.bit._1_4SECIE = 0;
		rtcinte_set.bit._1_8SECIE = 0;
		rtcinte_set.bit._1_32SECIE = 0;
		RTCINTE = rtcinte_set.reg;
	}
}


/************************************************************************//**
 * @brief  Set RTCA2 mode.
 *
 * @param   rtcaMode        12h or 24h mode.
 *  @li @c   C17_RTCA_12H   12H mode.
 *  @li @c   C17_RTCA_24H   24H mode.
 *
 * @param   Time zone            Set value.
 *  @li @c   C17_RTCA_ZONE_MAIN  Main Time Zone.
 *  @li @c   C17_RTCA_ZONE_2     Time Zone 2.
 *  @li @c   C17_RTCA_ZONE_3     Time Zone 3.
 *  @li @c   others              Nothing is done.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17setRtca2Mode(unsigned short rtcaMode, unsigned short zone)
{
	RTCCTLL_REG	rtcctll_set;
	RTCHUR1_REG	rtchur_set;
	ST_RTCA_BCD rtcBcd;
	unsigned short currentMode;

	/// Get Current Mode
	currentMode = c17getRtca2Mode();

	/// It doesn't do at all for the same mode.
	if(rtcaMode == currentMode){
		return EXIT_SUCCESS;
	}

	/// Get Current time.
	c17getRtca2Date(&rtcBcd, zone);

	/// The set value calculate.
	// 24H mode -> 12H mode.
	if (currentMode == C17_RTCA_24H) {
		if (rtcBcd.hour == 0) {						// 0o'clock(12AM).
			rtcBcd.hour = 0x12;
			rtcBcd.ampm = C17_RTCA_AM;
		} else if (rtcBcd.hour == 0x12) {			// 12o'clock(12PM).
			rtcBcd.ampm = C17_RTCA_PM;
		} else if (rtcBcd.hour >= 0x13) {			// 13 to 23o'clock(01PM to 11PM).
			rtcBcd.hour = numtobcd(bcdtonum(rtcBcd.hour) - 12);
			rtcBcd.ampm = C17_RTCA_PM;
		} else {									// 1 to 11o'clock(01AM to 11AM).
			rtcBcd.ampm = C17_RTCA_AM;
		}
	// 12H mode -> 24H mode.
	} else {
		// 12AM(0o'clock).
		if (rtcBcd.ampm == C17_RTCA_AM && rtcBcd.hour == 0x12) {
			rtcBcd.hour = 0;
		// 01PM to 11PM(13 to 23o'clock).
		} else 	if (rtcBcd.ampm == C17_RTCA_PM && rtcBcd.hour < 0x12) {
			rtcBcd.hour = numtobcd(bcdtonum(rtcBcd.hour) + 12);
		} else {
			// Non.
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 1;
		RTCCTLL = rtcctll_set.reg;
	}

	// Set mode.
	/* RTCCTL: RTC Control Register. */
	{
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTC24H = rtcaMode;
		RTCCTLL = rtcctll_set.reg;
	}

	/// Set hour.
	{
		switch(zone){
		case C17_RTCA_ZONE_MAIN:
			rtchur_set.reg = RTCHUR1;
			rtchur_set.bit.RTCAP = rtcBcd.ampm;
			rtchur_set.bit.RTCHH = rtcBcd.hour >> 4;
			rtchur_set.bit.RTCHL = rtcBcd.hour & 0x0f;
			RTCHUR1 = rtchur_set.reg;
			break;
		case C17_RTCA_ZONE_2:
			rtchur_set.reg = RTCHUR2;
			rtchur_set.bit.RTCAP = rtcBcd.ampm;
			rtchur_set.bit.RTCHH = rtcBcd.hour >> 4;
			rtchur_set.bit.RTCHL = rtcBcd.hour & 0x0f;
			RTCHUR2 = rtchur_set.reg;
			break;

		case C17_RTCA_ZONE_3:
			rtchur_set.reg = RTCHUR3;
			rtchur_set.bit.RTCAP = rtcBcd.ampm;
			rtchur_set.bit.RTCHH = rtcBcd.hour >> 4;
			rtchur_set.bit.RTCHL = rtcBcd.hour & 0x0f;
			RTCHUR3 = rtchur_set.reg;
			break;

		default:
			break;
		}
	}


	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 0;
		RTCCTLL = rtcctll_set.reg;
	}

	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief  Get RTCA2 mode.
 *
 * @param   none
 *
 * @retval  C17_RTCA_12H    12H mode.
 * @retval  C17_RTCA_24H    24H mode.
 ***************************************************************************/
unsigned short c17getRtca2Mode(void)
{
	RTCCTLL_REG	rtcctll_set;

	/// Check busy.
	{
		rtcctll_set.reg = RTCCTLL;
	}

	while(rtcctll_set.bit.RTCBSY == 1) {
		// Wait...
		{
			rtcctll_set.reg = RTCCTLL;
		}
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Hold RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 1;
		RTCCTLL = rtcctll_set.reg;
	}

	/* RTCCTL:RTC Control Register */
	{
		/// Resume RTCA count-up.
		rtcctll_set.reg = RTCCTLL;
		rtcctll_set.bit.RTCHLD = 0;
		RTCCTLL = rtcctll_set.reg;
	}

	return rtcctll_set.bit.RTC24H;
}


/************************************************************************//**
 * @brief   START SW.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startSw(void)
{
	RTCSWCTL_REG	rtcswctl_set;

	/* RTCSWCTL:RTC Stopwatch Control Register */
	{
		rtcswctl_set.reg = RTCSWCTL;
		rtcswctl_set.bit.SWRST = 1;
		rtcswctl_set.bit.SWRUN = 1;
		RTCSWCTL = rtcswctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   STOP SW.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopSw(void)
{
	RTCSWCTL_REG	rtcswctl_set;

	/* RTCSWCTL:RTC Stopwatch Control Register */
	{
		rtcswctl_set.reg = RTCSWCTL;
		rtcswctl_set.bit.SWRUN = 0;
		RTCSWCTL = rtcswctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Enable SW interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntSW(void)
{
	RTCINTF_REG	rtcintf_set;
	RTCINTE_REG	rtcinte_set;

	/// Clear interrupt flag.
	/* RTC Interrupt Flag Register. Interrupt Flag Register read & clear */
	{
		rtcintf_set.bit.SW1IF = 1;
		rtcintf_set.bit.SW10IF = 1;
		rtcintf_set.bit.SW100IF = 1;
		RTCINTF = rtcintf_set.reg;
	}

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		rtcinte_set.reg = RTCINTE;
	    /**
	     * @todo Change enable flag.
	     */
		rtcinte_set.bit.SW1IE = 1;
		rtcinte_set.bit.SW10IE = 1;
		rtcinte_set.bit.SW100IE = 1;
		RTCINTE = rtcinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Disable SW interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntSW(void)
{
	RTCINTE_REG	rtcinte_set;

	/* RTCINTE:RTC Interrupt Enable Register */
	{
		rtcinte_set.reg = RTCINTE;
		rtcinte_set.bit.SW1IE = 0;
		rtcinte_set.bit.SW10IE = 0;
		rtcinte_set.bit.SW100IE = 0;
		RTCINTE = rtcinte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Read SW count .
 *
 * @param   none
 *
 * @retval  BCD10 & BCD100
 ***************************************************************************/
unsigned short c17readSwCount(void)
{
	RTCSWCTL_REG	rtcswctl_set[3];

	/// Read RTCSWCTL register consecutively.
	rtcswctl_set[0].reg = RTCSWCTL;
	rtcswctl_set[1].reg = RTCSWCTL;
	rtcswctl_set[2].reg = RTCSWCTL;

	/// Compare read results and return their matched results.
	if(rtcswctl_set[0].reg == rtcswctl_set[1].reg){

		return ((rtcswctl_set[0].bit.BCD10 << 4) + rtcswctl_set[0].bit.BCD100);
	}
	else if(rtcswctl_set[1].reg == rtcswctl_set[2].reg){
		return ((rtcswctl_set[1].bit.BCD10 << 4) + rtcswctl_set[2].bit.BCD100);
	}
	else{
		return -1;
	}
}


static unsigned char numtobcd(unsigned char num)
{
	unsigned char bcd;

	bcd = (num / 10) << 4 ;
	bcd += (num  % 10);

	return bcd;
}


static unsigned char bcdtonum(unsigned char bcd)
{
	unsigned char num;

	num = (bcd >> 4) * 10;
	num += (bcd & 0x0f);

	return num;
}
#endif

