/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is CLG sample program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17W34/W35/W36 Clock Generator (CLG)
 *
 * @section     intro Introduction
 * It is a sample code for the clock source switch, and the system clock change.
 *
 * @subsection  step0   Step  0: Start OSC1, IOSC, OSC3.
 * @subsection  step1   Step  1: Halt. System Clock:IOSC
 * @subsection  step2   Step  2: Sleep. System Clock:IOSC.
 * @subsection  step3   Step  3: Change system clock from IOSC to OSC1.
 * @subsection  step4   Step  4: Halt. System Clock:OSC1.
 * @subsection  step5   Step  5: Sleep. System Clock:OSC1.
 * @subsection  step6   Step  6: Change system clock from OSC1 to IOSC.
 * @subsection  step7   Step  7: Change system clock from IOSC to OSC3.
 * @subsection  step8   Step  8: Halt. System Clock:OSC3.
 * @subsection  step9   Step  9: Sleep. System Clock:OSC3.
 * @subsection  step10  Step 10: Change system clock from OSC3 to IOSC.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17w34_w35_w36_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_t16.h"
#include "c17_init_config.h"


extern volatile unsigned int c17t16Ch0IntCount;
/* --- global variable --- */

int main_clg(void);

/************************************************************************//**
 * main function.
 *
 * @brief   1, halt until T16 interrupt. (OSC1/IOSC/OSC3)
 *
 *          2, sleep until T16 interrupt. (OSC1/IOSC/OSC3)
 *
 *          3, change the system clock. (OSC1/IOSC/OSC3)
 *
 *          4, FOUT enable. (OSC1/IOSC/OSC3)
 *
 * @return  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_clg(void)
{
	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();

	/// Initialize and start OSC1.
	c17startClgOSC1();

	/// Initialize and start OSC3.
	c17stopClgOSC3();
	c17startClgIntOSC3(C17_CLG_OSC3_4MHZ);

	/// Continue operation state before SLEEP
	c17setOsc3OpeInSleep(C17_CLG_CONTINUE);
	c17setOsc1OpeInSleep(C17_CLG_CONTINUE);
	c17setIoscOpeInSleep(C17_CLG_CONTINUE);

	/* <<< CLG demonstration start >>> */

	/// Clear Interruption counter.
	c17t16Ch0IntCount = 0;

	/**
	 * Clock Source = IOSC(700kHz)
	 * Clock Division = 1/32
	 * Ft = 1sec
	 * TR = 1[sec] * f((700kHz/32) -1 = 21874
	 */
	/// Initialize  T16 CH.0 timer.
	c17setT16(0, C17_T16_CLKSRC_IOSC, C17_T16_CLKDIV_32, C17_T16_MODE_REPEAT);

	/// Set Reload Value T16 CH.0
	c17setCountT16(0, 21874);

	/// Initialize the interrupt.
	ILVT16_0 = INTERRUPT_LEVEL_T16_0;				/// - Set interrupt level.

	/// T16 CH.0 Interrupt Enable
	c17enableIntT16(0);

	/// Start T16 CH.0 timer.
	c17startT16(0);

	/// FOUT = IOSC Clock
	c17enableFout(C17_CLG_FOUT_IOSC,C17_CLG_FOUT_CLKDIV_1);

	/// IOSC Now halt.
	while(c17t16Ch0IntCount < 2){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("halt");
		asm("nop");

	}

	/// IOSC Now sleep.
	while(c17t16Ch0IntCount < 4){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("slp");
		asm("nop");
	}

	/// Disable FOUT
	c17disableFout();


	/// Change from IOSC to OSC1
	c17startClgOSC1();
	c17switchClgSystemClockOSC1();

	/// FOUT = OSC1 Clock
	c17enableFout(C17_CLG_FOUT_OSC1,C17_CLG_FOUT_CLKDIV_1);

	/// OSC1 Now halt.
	while(c17t16Ch0IntCount < 6){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("halt");
		asm("nop");
	}

	/// OSC1 Now sleep.
	while(c17t16Ch0IntCount < 8){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("slp");
		asm("nop");
	}
	/// Wake UP OSC1->IOSC


	/// Disable FOUT
	c17disableFout();


	/// Change from IOSC to OSC3.
	/// the system clock set in OSC3.
	c17startClgIntOSC3(C17_CLG_OSC3_4MHZ);
	c17switchClgSystemClockOSC3();

	/// FOUT = OSC3 Clock
	c17enableFout(C17_CLG_FOUT_OSC3,C17_CLG_FOUT_CLKDIV_1);

	/// OSC3 Now halt.
	while(c17t16Ch0IntCount < 10){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("halt");
		asm("nop");
	}

	/// OSC3 Now sleep.
	while(c17t16Ch0IntCount < 12){
		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm("slp");
		asm("nop");
	}
	/// Wake UP OSC3->IOSC

	/// stop OSC3
	c17stopClgOSC3();

	/// Disable FOUT
	c17disableFout();


	/// Finish CLG demonstration.
	return EXIT_SUCCESS;

}

