/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is LCD32B sample program for S1C17W34/W35/W36 demonstration.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17W34/W35/W36 LCD32B
 *
 * @section     intro Introduction
 * Sample code turns on the LCD display.
 *
 * @subsection  step1   Step  1: Start OSC1.
 * @subsection  step2   Step  2: LCD register initialize and LCD display on.
 * @subsection  step3   Step  3: Start RTCA(1-second interrupt).
 * @subsection  step4   Step  4: Turn on LCD display by all on.
 * @subsection  step5   Step  5: Turn on LCD display by all off.
 * @subsection  step6   Step  6: Turn on LCD display area 0.
 *                               Set display area 0 data by checker-board pattern(small).
 *                               Turn on LCD display by normal display.
 * @subsection	step7   Step  7: Set LCD driver's inverted display.
 * @subsection	step8   Step  8: Turn on LCD display by normal display.
 *                               Set display area 1 data by checker-board pattern(medium).
 *                               Turn on LCD display area 1.
 * @subsection	step9   Step  9: Set LCD driver's inverted display.
 * @subsection	step10  Step 10: Turn on LCD display by normal display.
 *                               Set display area 0 data by checker-board pattern(large).
 *                               Turn on LCD display area 0.
 * @subsection	step11  Step 11: Set LCD driver's inverted display.
 * @subsection	step12  Step 12: Turn off LCD display by display off.
 *                               End sample program.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17w34_w35_w36_sample" project.
 *
 ************************************************************************* */
#include <stdlib.h>
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_rtca2.h"
#include "c17_lcd32b.h"
#include "c17_init_config.h"

#if defined(C17_PE_LCD32B)

/* --- definition --- */
/**
 * @todo change display switch time[sec].
 */
#define LCD_DISPLAY_SWITCH_TIME	(2)

/* --- global variable --- */
extern volatile unsigned int c17rtca2IntCount;

/* --- function declaration --- */

int main_lcd32b(void);


/************************************************************************//**
 * main function.
 *
 * @brief   1, LCD register initialize.
 *
 *          2, LCD display on.
 *
 *          3, Stop after the display off.
 *
 * @return 	EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_lcd32b(void)
{
	volatile unsigned int secCount = 0;
	volatile unsigned int secOld   = 0;

	/* <<< LCD32B driver demonstration start >>> */

	/// Initialize and start OSC1.
	c17startClgOSC1();

	/// Initialize LCD.
	c17initLcd32b();

	/// Display on.
	c17onLcd32bDisplay();

	/// Initialize and start RTCA.
	c17initRtca2();

	/// RTCA Interrupt Enable.
	c17enableIntRtca2();

	/// Clear Interruption counter.
	c17rtca2IntCount = 0;

	/// Start RTCA.
	c17startRtca2();

	/// Display LCD.
	while(secCount < (10 * LCD_DISPLAY_SWITCH_TIME)){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

		secOld = secCount;
		secCount = c17rtca2IntCount;

		/// Nothing is done for the same time.
		if (secOld == secCount) {
			continue;
		}

		switch (secCount) {
		case 1 * LCD_DISPLAY_SWITCH_TIME:
			/// The LCD display set the state to "all on".
			c17setLcd32bDisplayState(C17_LCD_STATE_ALL_ON);
			break;

		case 2 * LCD_DISPLAY_SWITCH_TIME:
			/// The LCD display set the state to "all off".
			c17setLcd32bDisplayState(C17_LCD_STATE_ALL_OFF);
			break;

		case 3 * LCD_DISPLAY_SWITCH_TIME:
			/// Set display area 0 data by checker-board pattern(small).
			c17setLcd32bCheckerboardPatternSmall(C17_LCD_DISPLAY_AREA0);
			/// Set LCD driver's display area 0.
			c17setLcd32bDisplayArea(C17_LCD_DISPLAY_AREA0);
			/// The LCD display set the state to "Normal display".
			c17setLcd32bDisplayState(C17_LCD_STATE_NORMAL);
			break;

		case 4 * LCD_DISPLAY_SWITCH_TIME:
			/// Set LCD driver's inverted display.
			c17setLcd32bDisplayInverted(C17_LCD_DISPLAY_INVERTED);
			break;

		case 5 * LCD_DISPLAY_SWITCH_TIME:
			/// Set display area 1 data by checker-board pattern(medium).
			c17setLcd32bCheckerboardPatternMedium(C17_LCD_DISPLAY_AREA1);
			/// Set LCD driver's display area 1.
			c17setLcd32bDisplayArea(C17_LCD_DISPLAY_AREA1);
			/// Set LCD driver's normal display.
			c17setLcd32bDisplayInverted(C17_LCD_DISPLAY_NORMAL);
			break;

		case 6 * LCD_DISPLAY_SWITCH_TIME:
			/// Set LCD driver's inverted display.
			c17setLcd32bDisplayInverted(C17_LCD_DISPLAY_INVERTED);
			break;

		case 7 * LCD_DISPLAY_SWITCH_TIME:
			/// Set display area 0 data by checker-board pattern(large).
			c17setLcd32bCheckerboardPatternLarge(C17_LCD_DISPLAY_AREA0);
			/// Set LCD driver's display area 0.
			c17setLcd32bDisplayArea(C17_LCD_DISPLAY_AREA0);
			/// Set LCD driver's normal display.
			c17setLcd32bDisplayInverted(C17_LCD_DISPLAY_NORMAL);
			break;

		case 8 * LCD_DISPLAY_SWITCH_TIME:
			/// Set LCD driver's inverted display.
			c17setLcd32bDisplayInverted(C17_LCD_DISPLAY_INVERTED);
			break;

		case 9 * LCD_DISPLAY_SWITCH_TIME:
			/// The LCD display state set to "Display off".
			c17setLcd32bDisplayState(C17_LCD_STATE_OFF);
			break;

		default:
			/// Non.
			break;
		}
	}

	/// Display off.
	c17offLcd32bDisplay();

	/// Stop RTCA.
	c17stopRtca2();

	/* <<< LCD32B driver demonstration finish >>> */

	return EXIT_SUCCESS;
}
#endif

