/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is RTCA2 sample program for S1C17W34/W35/W36 demonstration.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17W34/W35/W36 Real Time Clock (RTCA2)
 *
 * @section     intro Introduction
 * Sample code for generating interruption at intervals of 1sec after setting time to RTCA,
 * and execute theoretical regulation.
 * @subsection  step1   Step 1: Start OSC1 and Stop IOSC.
 * @subsection  step2   Step 2: Start RTCA(1-second interrupt and theoretical regulation).
 * @subsection  step3   Step 3: Wait RTCA 1sec Interrupt.
 *                      waits until 1sec interrupt occurs 10 times.
 * @subsection  step4   Step 4: The set value at time is acquired.
 *                      End sample program(halt).
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17w34_w35_w36_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_rtca2.h"
#include "c17_init_config.h"

#if defined(C17_PE_RTCA)

extern volatile unsigned int c17rtca2IntCount;
extern volatile unsigned int c17swIntCount;

/* --- definition --- */

ST_RTCA_BCD rtcBcd;

int main_rtca2(void);


/****************************************************************************
 * main function.
 *
 * @brief
 *          Main function starts from real-time clock sample.
 *          First,RTCA register is initialized.
 *			Second,A setRtca function is called,
 *			RTCA information is set up and RTCA is started.
 * 			Next,It waits until 1sec interrupt occurs 10 times.
 *			then,RTCA is stopped.
 *
 *			Latter half of main function is stopwatch sample.
 *			Firstly, initializes stopwatch.
 *			Secondarily, starts stopwatch and waits till 10 interrupts of 1Hz counts.
 *			Then, reads 10Hz and 100Hz digits of stopwatch counts and stops stopwatch.
 *
 * @retval	EXIT_SUCCESS	Success.
 ***************************************************************************/
int main_rtca2(void)
{

	/* <<< Real Time Clock demonstration start >>> */

	/// Initialize and start OSC1.
	c17startClgOSC1();								// start OSC1.

	/// Initialize RTCA.
	c17initRtca2();									// initialize RTCA.

	/// Set RTCA date.
	rtcBcd.year = 0x16;								// Year = 2016
	rtcBcd.month = 0x12;							// Month  = 12.
	rtcBcd.day = 0x31;								// Day    = 31.
	rtcBcd.hour = 0x23;								// Hour   = 23.
	rtcBcd.minute = 0x59;							// Minute = 59.
	rtcBcd.second = 0x51;							// Second = 51.
	rtcBcd.week = C17_RTCA_SAT;
	rtcBcd.ampm = C17_RTCA_PM;
	c17setRtca2Date(&rtcBcd, C17_RTCA_ZONE_MAIN);

	/// Enable Interrupt
	c17enableIntRtca2();

	/// Start RTCA
	c17startRtca2();

	/// Check RTCA interrupt count.
	while(c17rtca2IntCount < 10){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ( "halt" );
		asm ( "nop" );
	}

	//12H mode
	c17setRtca2Mode(C17_RTCA_12H, C17_RTCA_ZONE_MAIN);
	/// Get RTCA date.
	c17getRtca2Date(&rtcBcd, C17_RTCA_ZONE_MAIN);

	/// Initialize stop watch.
	c17startSw();
	c17enableIntSW();

	while(c17swIntCount <10){
		// Set halt mode.
		asm ( "halt" );
		asm ( "nop" );
	}

	c17stopSw();
	c17disableIntSW();

	c17stopRtca2();

	/* <<< Real Time Clock demonstration finish >>> */

	return EXIT_SUCCESS;

}
#endif

