/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is 16-bit timers sample program for S1C17W34/W35/W36 demonstration.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
/************************************************************************//**
 * @mainpage    S1C17W34/W35/W36 T16
 *
 * @section     intro Introduction
 * Sample code gets counter value of timer by T16 timer interrupt.
 * When wait T16 timer interrupt, the code changes Halt mode of CPU,
 * reduce consumption electricity.
 *
 * @subsection  step1   Step 1: Initialize T16 timer.
 *                      Initialize T16 timer.
 * @subsection  step2   Step 2: Start T16 timer.
 *                      Start T16 timer interrupt.
 *                      Change Halt mode of CPU.
 * @subsection  step3   Step 3: Get T16 timer interrupt count.
 *                      When the T16 timer interruption generate, get the
 *                      interrupt count.
 *                      Keep the interrupt count to internal variable.
 *                      Change Halt mode of CPU.
 * @subsection  step4   Step 4: Repeat 1000 times.
 *                      Repeat T16 timer interrupt 1000 times.
 *                      1sec x 10 = 10sec.
 * @subsection  step5   Step 5:Stop T16 timer.
 *                      Stop T16 timer interrupt and the timer are stopped.
 *                      End sample program.
 *
 * @section     install Installation
 * Start the IDE and import the "s1c17w34_w35_w36_sample" project.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "c17_misc.h"
#include "c17_t16.h"
#include "c17_init_config.h"

#if defined(C17_PE_T16)

extern volatile unsigned int c17t16Ch0IntCount;

/* --- function declaration --- */

int main_t16(void);


/************************************************************************//**
 * main function.
 *
 * @brief   1, initialize T16 timer.
 *
 *          2, set the counter of the T16 timer.
 *
 *          3, initialize and start T16 timer interrupt.
 *
 *          4, it waits until underflow interrupt occurs 1000 times.
 *
 *          5, stop T16 timer.
 *
 * @return  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_t16(void)
{
	/* <<< 16-bit Timers demonstration start >>> */

	/// Clear Interruption counter.
	c17t16Ch0IntCount = 0;

	/**
	 * Clock Source=IOSC(700KHz)
	 * Clock Division = 1/32
	 * Ft = 1sec
	 * TR = 1[sec] * f(700KHz/32) -1 = 21874
	 */
	/// Initialize T16 CH.0 timer.
	c17initT16();

	/// Set Reload Value T16 CH.0
	c17setCountT16(0, 21874);

	/// T16 CH.0 Interrupt Enable
	c17enableIntT16(0);

	/// Start T16 CH.0 timer.
	c17startT16(0);

	/// Get T16 CH.0 interrupt count.
	while(c17t16Ch0IntCount < 10){

		// Set halt mode.
		// Nop is necessary when the main program is not executed after
		// halt is	released, and it diverges to the server blade.
		asm ("halt");
		asm ("nop");

	}

	/// Stop T16 CH.0 timer.
	c17stopT16(0);

	/// Disable T16 CH.0 timer.
	c17disableT16(0);

	/* <<< 16-bit Timers demonstration finish >>> */

	return EXIT_SUCCESS;
}
#endif

