/************************************************************************//**
 *
 * @file    i2c_reg.h
 *
 * @brief   Definition of the I2C Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef I2C_REG_H__
#define I2C_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// I2C Ch.N Clock Control Register
typedef union I2CNCLK_REG_TAG {
    unsigned short  reg;
    struct I2CNCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the I2C.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the I2C.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the I2C operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[1:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |     0x3     |1/8     |1/1     |1/8     |1/1     |
         * +-------------+--------+        +--------+        +
         * |     0x2     |1/4     |        |1/4     |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |1/2     |        |1/2     |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |1/1     |        |1/1     |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the I2C operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the I2C.
         * .CLKDIV[1:0]
         *  These bits select the division ratio of the I2C operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * |             |  0x0   |   0x1  |  0x2   |  0x3   |
         * | CLKDIV[1:0] +--------+--------+--------+--------+
         * |             |        |        |        |        |
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * |             |        |        |        |        |
         * +-------------+--------+--------+--------+--------+
         * |     0x3     |1/8     |1/1     |1/8     |1/1     |
         * +-------------+--------+        +--------+        +
         * |     0x2     |1/4     |        |1/4     |        |
         * +-------------+--------+        +--------+        +
         * |     0x1     |1/2     |        |1/2     |        |
         * +-------------+--------+        +--------+        +
         * |     0x0     |1/1     |        |1/1     |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * This bit sets whether the I2C operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the I2C operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNCLK_REG;

/// I2C Ch.N Mode Register
typedef union I2CNMOD_REG_TAG {
    unsigned short  reg;
    struct I2CNMOD_REG_BIT_TAG {
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit sets whether to respond to master general calls in slave mode or not.
         * .GCEN
         *  This bit sets whether to respond to master general calls in slave mode or not.
         *    1 (R/W): Respond to general calls.
         *    0 (R/W): Do not respond to general calls.
         */
        unsigned short      GCEN        : 1;
        /**
         * This bit sets the number of own address bits for slave mode.
         * .OADR10
         *  This bit sets the number of own address bits for slave mode.
         *    1 (R/W): 10-bit address
         *    0 (R/W): 7-bit address
         */
        unsigned short      OADR10      : 1;
        unsigned short      reserved_2  :13;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNMOD_REG;

/// I2C Ch.N Baud-Rate Register
typedef union I2CNBR_REG_TAG {
    unsigned short  reg;
    struct I2CNBR_REG_BIT_TAG {
        /**
         * These bits set the I2C Ch.0 transfer rate for master mode.
         * .BRT[6:0]
         *  These bits set the I2C Ch.n transfer rate for master mode. For more information, refer to <Baud Rate Generator>.
         */
        unsigned short      BRT         : 7;
        unsigned short      reserved_1  : 9;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNBR_REG;

/// I2C Ch.N Own Address Register
typedef union I2CNOADR_REG_TAG {
    unsigned short  reg;
    struct I2CNOADR_REG_BIT_TAG {
        /**
         * These bits set the own address for slave mode.
         * .OADR[9:0]
         *  These bits set the own address for slave mode.
         *  The I2CnOADR.OADR[9:0] bits are effective in 10-bit address mode (I2CnMOD.OADR10 bit = 1),
         *  or the I2CnOADR.OADR[6:0] bits are effective in 7-bit address mode (I2CnMOD.OADR10 bit = 0).
         */
        unsigned short      OADR        :10;
        unsigned short      reserved_1  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNOADR_REG;

/// I2C Ch.N Control Register
typedef union I2CNCTL_REG_TAG {
    unsigned short  reg;
    struct I2CNCTL_REG_BIT_TAG {
        /**
         * This bit enables the I2C operations.
         * .MODEN
         *  This bit enables the I2C operations.
         *    1 (R/W): Enable I2C operations (The operating clock is supplied.)
         *    0 (R/W): Disable I2C operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        /**
         * This bit issues software reset to the I2C.
         * .SFTRST
         *  This bit issues software reset to the I2C.
         *    1 (W): Issue software reset
         *    0 (W): Ineffective
         *    1 (R): Software reset is executing.
         *    0 (R): Software reset has finished. (During normal operation)
         */
        unsigned short      SFTRST      : 1;
        /**
         * This bit issues a START condition in master mode.
         * .TXSTART
         *  This bit issues a START condition in master mode. This bit is ineffective in slave mode.
         *    1 (W): Issue a START condition.
         *    0 (W): Ineffective
         *    1 (R): On standby or during generating a START condition
         *    0 (R): START condition has been generated.
         */
        unsigned short      TXSTART     : 1;
        /**
         * This bit issues a STOP condition in master mode.
         * .TXSTOP
         *  This bit issues a STOP condition in master mode. This bit is ineffective in slave mode.
         *    1 (W): Issue a STOP condition.
         *    0 (W): Ineffective
         *    1 (R): On standby or during generating a STOP condition
         *    0 (R): STOP condition has been generated.
         */
        unsigned short      TXSTOP      : 1;
        /**
         * This bit issues a request for sending a NACK at the next responding.
         * .TXNACK
         *  This bit issues a request for sending a NACK at the next responding.
         *    1 (W): Issue a NACK.
         *    0 (W): Ineffective
         *    1 (R): On standby or during sending a NACK
         *    0 (R): NACK has been sent.
         */
        unsigned short      TXNACK      : 1;
        /**
         * This bit selects the I2C Ch.n operating mode.
         * .MST
         *  This bit selects the I2C Ch.n operating mode.
         *    1 (R/W): Master mode
         *    0 (R/W): Slave mode
         */
        unsigned short      MST         : 1;
        unsigned short      reserved_1  :10;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNCTL_REG;

/// I2C Ch.N Transmit Data Register
typedef union I2CNTXD_REG_TAG {
    unsigned short  reg;
    struct I2CNTXD_REG_BIT_TAG {
        /**
         * Data can be written to the transmit data buffer through these bits.
         * .TXD[7:0]
         *  Data can be written to the transmit data buffer through these bits. Make sure the I2CnINTF.TBEIF bit
         *  is set to 1 before writing data.
         */
        unsigned short      TXD         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNTXD_REG;

/// I2C Ch.N Receive Data Register
typedef union I2CNRXD_REG_TAG {
    unsigned short  reg;
    struct I2CNRXD_REG_BIT_TAG {
        /**
         * The receive data buffer can be read through these bits.
         * .RXD[7:0]
         *  The receive data buffer can be read through these bits.
         */
        unsigned short      RXD         : 8;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNRXD_REG;

/// I2C Ch.N Status and Interrupt Flag Register
typedef union I2CNINTF_REG_TAG {
    unsigned short  reg;
    struct I2CNINTF_REG_BIT_TAG {
        /**
         * This bit indicate Transmit buffer empty interrupt cause occurrence status.
         * .TBEIF
         *  These bits indicate the I2C interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .TBEIF bit: Transmit buffer empty interrupt
         */
        unsigned short      TBEIF       : 1;
        /**
         * This bit indicate Receive buffer full interrupt cause occurrence status.
         * .RBFIF
         *  Receive buffer full interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      RBFIF       : 1;
        /**
         * This bit indicate Error detection interrupt cause occurrence status.
         * .ERRIF
         *  Error detection interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      ERRIF       : 1;
        /**
         * This bit indicate START condition interrupt cause occurrence status.
         * .STARTIF
         *  START condition interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      STARTIF     : 1;
        /**
         * This bit indicate STOP condition interrupt cause occurrence status.
         * .STOPIF
         *  STOP condition interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      STOPIF      : 1;
        /**
         * This bit indicate NACK reception interrupt cause occurrence status.
         * .NACKIF
         *  NACK reception interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      NACKIF      : 1;
        /**
         * This bit indicate General call address reception interrupt cause occurrence status.
         * .GCIF
         *  General call address reception interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      GCIF        : 1;
        /**
         * This bit indicate End of transfer interrupt cause occurrence status.
         * .BYTEENDIF
         *  End of transfer interrupt
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      BYTEENDIF   : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit indicates whether the I2C is set in transmission mode or not.
         * .TR
         *  This bit indicates whether the I2C is set in transmission mode or not.
         *    1 (R): Transmission mode
         *    0 (R): Reception mode
         */
        unsigned short      TR          : 1;
        /**
         * This bit indicates that the I2C bus is placed into busy status.
         * .BSY
         *  This bit indicates that the I2C bus is placed into busy status.
         *    1 (R): I2C bus busy
         *    0 (R): I2C bus free
         */
        unsigned short      BSY         : 1;
        /**
         * This bit indicates that SCL is set to low level.
         * .SCLLOW
         *  This bit indicates that SCL is set to low level.
         *    1 (R): SCL = Low level
         *    0 (R): SCL = High level
         */
        unsigned short      SCLLOW      : 1;
        /**
         * This bit indicates that SDA is set to low level.
         * .SDALOW
         *  This bit indicates that SDA is set to low level.
         *    1 (R): SDA = Low level
         *    0 (R): SDA = High level
         */
        unsigned short      SDALOW      : 1;
        unsigned short      reserved_2  : 3;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNINTF_REG;

/// I2C Ch.N Interrupt Enable Register
typedef union I2CNINTE_REG_TAG {
    unsigned short  reg;
    struct I2CNINTE_REG_BIT_TAG {
        /**
         * This bit enable Transmit buffer empty interrupt.
         * .TBEIE
         *  These bits enable I2C interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .TBEIE bit: Transmit buffer empty interrupt
         */
        unsigned short      TBEIE       : 1;
        /**
         * This bit enable Receive buffer full interrupt.
         * .RBFIE
         *  Receive buffer full interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      RBFIE       : 1;
        /**
         * This bit enable Error detection interrupt.
         * .ERRIE
         *  Error detection interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      ERRIE       : 1;
        /**
         * This bit enable START condition interrupt.
         * .STARTIE
         *  START condition interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      STARTIE     : 1;
        /**
         * This bit enable STOP condition interrupt.
         * .STOPIE
         *  STOP condition interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      STOPIE      : 1;
        /**
         * This bit enable NACK reception interrupt.
         * .NACKIE
         *  NACK reception interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      NACKIE      : 1;
        /**
         * This bit enable General call address reception interrupt.
         * .GCIE
         *  General call address reception interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      GCIE        : 1;
        /**
         * This bit enable End of transfer interrupt.
         * .BYTEENDIE
         *  End of transfer interrupt
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         */
        unsigned short      BYTEENDIE   : 1;
        unsigned short      reserved_1  : 8;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} I2CNINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // I2C_REG_H__
