/************************************************************************//**
 *
 * @file    c17_lcd32b.c
 *
 * @brief   This is LCD32B driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_lcd32b.h"
#include "c17_init_config.h"

#if defined(C17_PE_LCD32B)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector22_handler, c17intLcd32b);		// LCD32B interrupt
#endif /* CRT0_LINKED */


/* --- function declaration --- */

/************************************************************************//**
 * @brief   This function initializes LCD32B driver.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initLcd32b(void)
{
	/// Disable LCD32B.
	c17offLcd32bDisplay();

	/// LCD32B Setting
	if((LCD32B_INIT & 0x20) == 0x20)
	{
		/// Initialize LCD32B.
		c17setLcd32b((LCD32B_INIT & 0x18)>>3, (LCD32B_INIT & 0x0F));

		/// Initialize the interrupt.
		ILVLCD32B = INTERRUPT_LEVEL_LCD32B;				/// - Set interrupt level.
	}
}


/************************************************************************//**
 * @brief   Enable LCD32B interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntLcd32b(void)
{
 	LCD32INTF_REG	lcd32intf_set;
 	LCD32INTE_REG	lcd32inte_set;

	/// Clear interrupt flags.
	/* LCD32INTF:LCD32B Interrupt Flag Register. */
	{
		lcd32intf_set.bit.FRMIF = 1;
		LCD32INTF = lcd32intf_set.reg;
	}

	/// Disable LCD32B frame interrupts.
	/* LCD32INTE:LCD32B Interrupt Enable Register. */
	{
		lcd32inte_set.bit.FRMIE = 1;
		LCD32INTE = lcd32inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Disable LCD32B interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntLcd32b(void)
{
 	LCD32INTF_REG	lcd32intf_set;
 	LCD32INTE_REG	lcd32inte_set;

	/// Clear interrupt flags.
	/* LCD32INTF:LCD32B Interrupt Flag Register. */
	{
		lcd32intf_set.bit.FRMIF = 1;
		LCD32INTF = lcd32intf_set.reg;
	}

	/// Disable LCD32B frame interrupts.
	/* LCD32INTE:LCD32B Interrupt Enable Register. */
	{
		lcd32inte_set.bit.FRMIE = 0;
		LCD32INTE = lcd32inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   The display turn on.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17onLcd32bDisplay(void)
{
 	LCD32CTL_REG	lcd32ctl_set;
 	LCD32DSP_REG	lcd32dsp_set;

	/* LCD32CTL:LCD32B Control Register */
	{
		lcd32ctl_set.reg = LCD32CTL;
		lcd32ctl_set.bit.MODEN = 1;			/// - LCD32B enable
		lcd32ctl_set.bit.LCDDIS = 1;		/// - LCD32B LCDDIS enable
		LCD32CTL = lcd32ctl_set.reg;
	}

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		lcd32dsp_set.reg = LCD32DSP;
		lcd32dsp_set.bit.DSPC = 1;			/// The LCD display state set to "normal display".
		LCD32DSP = lcd32dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   The display turn off.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17offLcd32bDisplay(void)
{
 	LCD32CTL_REG	lcd32ctl_set;
 	LCD32DSP_REG	lcd32dsp_set;

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		lcd32dsp_set.reg = LCD32DSP;
		lcd32dsp_set.bit.DSPC = 0;			/// The LCD display state set to "display off".
		LCD32DSP = lcd32dsp_set.reg;
	}

	/* LCD32CTL:LCD32B Control Register */
	{
		lcd32ctl_set.reg = LCD32CTL;
		lcd32ctl_set.bit.MODEN = 0;			/// - LCD32B disable
		LCD32CTL = lcd32ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function changes LCD driver's state.
 *
 * @param   state
 *  @li @c  C17_LCD_STATE_ALL_OFF   Mode : All off.
 *  @li @c  C17_LCD_STATE_ALL_ON    Mode : All on.
 *  @li @c  C17_LCD_STATE_NORMAL    Mode : Normal display.
 *  @li @c  C17_LCD_STATE_OFF       Mode : Display off.
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd32bDisplayState(unsigned char state)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		lcd32dsp_set.reg = LCD32DSP;
		lcd32dsp_set.bit.DSPC = state;		/// The LCD display state set.
		LCD32DSP = lcd32dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets LCD driver's state.
 *
 * @param   none
 *
 * @retval  C17_LCD_STATE_ALL_OFF   Mode : All off.
 * @retval  C17_LCD_STATE_ALL_ON    Mode : All on.
 * @retval  C17_LCD_STATE_NORMAL    Mode : Normal display.
 * @retval  C17_LCD_STATE_OFF       Mode : Display off.
 ***************************************************************************/
unsigned char c17getLcd32bDisplayState(void)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/// LCD32B Display Control Register
 	lcd32dsp_set.reg = LCD32DSP;

	/// LCD display mode.
	return lcd32dsp_set.bit.DSPC;
}


/************************************************************************//**
 * @brief   This function changes LCD driver's display area.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval	none
 ***************************************************************************/
void c17setLcd32bDisplayArea(unsigned char area)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		lcd32dsp_set.reg = LCD32DSP;
		lcd32dsp_set.bit.DSPAR = area;		/// The LCD display area set.
		LCD32DSP = lcd32dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets LCD driver's display area.
 *
 * @param   none
 *
 * @retval  C17_LCD_DISPLAY_AREA0   display area 0.
 * @retval  C17_LCD_DISPLAY_AREA1   display area 1.
 ***************************************************************************/
unsigned char c17getLcd32bDisplayArea(void)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/// LCD32B Display Control Register
 	lcd32dsp_set.reg = LCD32DSP;

	/// LCD display area.
	return lcd32dsp_set.bit.DSPAR;
}


/************************************************************************//**
 * @brief   This function changes LCD driver's display.
 *
 * @param   display
 *  @li @c  C17_LCD_DISPLAY_INVERTED    Inverted display.
 *  @li @c  C17_LCD_DISPLAY_NORMAL      Normal display.
 *
 * @retval  none
 ****************************************************************************/
void c17setLcd32bDisplayInverted(unsigned char display)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		lcd32dsp_set.reg = LCD32DSP;
		lcd32dsp_set.bit.DSPREV = display;		/// The LCD display black/white inversion set.
		LCD32DSP = lcd32dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets LCD driver's display.
 *
 * @param   none
 *
 * @retval  C17_LCD_DISPLAY_INVERTED    Inverted display.
 * @retval  C17_LCD_DISPLAY_NORMAL      Normal display.
 ***************************************************************************/
unsigned char c17getLcd32bDisplayInverted(void)
{
 	LCD32DSP_REG	lcd32dsp_set;

	/// LCD32B Display Control Register
 	lcd32dsp_set.reg = LCD32DSP;

	/// LCD display black/white inversion.
	return lcd32dsp_set.bit.DSPREV;
}


/************************************************************************//**
 * @brief   This function clear LCD driver's display data.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval  none
 ***************************************************************************/
void c17clearLcd32bDisplayData(unsigned char area)
{
	unsigned short i;
	volatile unsigned char *segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA0;

	if (area == C17_LCD_DISPLAY_AREA1) {
		segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA1;
	}

	for (i = 0; i < LCD32B_SEGRAM_SIZE; i++) {
		segram[ i ] = 0x00;
	}
}


/************************************************************************//**
 * @brief   This function set checker-board pattern(small) for display.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd32bCheckerboardPatternSmall(unsigned char area)
{
	unsigned short i;
	volatile unsigned char *segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA0;

	if (area == C17_LCD_DISPLAY_AREA1) {
		segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA1;
	}

	for (i = 0; i < LCD32B_SEGRAM_SIZE; i++) {
		if (i % 8 < 4) {
			segram[ i ] = 0x55;
		} else {
			segram[ i ] = 0xAA;
		}
	}
}


/************************************************************************//**
 * @brief   This function set checker-board pattern(medium) for display.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd32bCheckerboardPatternMedium(unsigned char area)
{
	unsigned short i;
	volatile unsigned char *segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA0;

	if (area == C17_LCD_DISPLAY_AREA1) {
		segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA1;
	}

	for (i = 0; i < LCD32B_SEGRAM_SIZE; i++) {
		if (i % 16 < 8) {
			segram[ i ] = 0x33;
		} else {
			segram[ i ] = 0xCC;
		}
	}
}


/************************************************************************//**
 * @brief   This function set checker-board pattern(large) for display.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd32bCheckerboardPatternLarge(unsigned char area)
{
	unsigned short i;
	volatile unsigned char *segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA0;

	if (area == C17_LCD_DISPLAY_AREA1) {
		segram = (unsigned char *)LCD32B_SEGRAM_TOP_ADDR_AREA1;
	}

	for (i = 0; i < LCD32B_SEGRAM_SIZE; i = i + 2) {
		if (i % 32 < 12) {
			segram[ i + 0 ] = 0xC7;
			segram[ i + 1 ] = 0x71;
		} else {
			segram[ i + 0 ] = 0x38;
			segram[ i + 1 ] = 0x8E;
		}
	}
}


/************************************************************************//**
 * @brief   This function sets LCD32B.
 *
 * @param   ledcClkSrc      Clock Source.
 *  @li @c  C17_LEDC_IOSC
 *  @li @c  C17_LEDC_OSC1
 *  @li @c  C17_LEDC_OSC3
 *  @li @c  C17_LEDC_EXT
 * @param   ledcClkDiv      Clock Division
 *  @li @c  C17_LEDC_CLKDIV_1
 *  @li @c  C17_LEDC_CLKDIV_4
 *  @li @c  C17_LEDC_CLKDIV_8
 *  @li @c  C17_LEDC_CLKDIV_16
 *  @li @c  C17_LEDC_CLKDIV_32
 *  @li @c  C17_LEDC_CLKDIV_64
 *  @li @c  C17_LEDC_CLKDIV_128
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd32b(unsigned short lcdClkSrc, unsigned short lcdClkDiv)
{

	LCD32CLK_REG	lcd32clk_set;

 	LCD32TIM1_REG	lcd32tim1_set;
 	LCD32TIM2_REG	lcd32tim2_set;

 	LCD32PWR_REG	lcd32pwr_set;

 	LCD32DSP_REG	lcd32dsp_set;

 	LCD32COMC0_REG	lcd32comc0_set;
 	LCD32COMC1_REG	lcd32comc1_set;

 	LCD32INTF_REG	lcd32intf_set;
 	LCD32INTE_REG	lcd32inte_set;


	/// Initialize LEDC ports.

	/// - LCD32B Driver (COM8 to COM12)
	/// - LCD32B Driver (SEG59 to SEG63 / COM13 to COM15)

	/* LCD32CLK:LCD32B Clock Control Register */
	{
		lcd32clk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		lcd32clk_set.bit.CLKDIV = lcdClkDiv;	/// - Select the division ratio.
		lcd32clk_set.bit.CLKSRC = lcdClkSrc;	/// - Select the clock source of the LCD32B.
		LCD32CLK = lcd32clk_set.reg;
	}

	/* LCD32TIM1:LCD32B Timing Control Register 1. */
	{
		/**
	     * @todo Change LCD32B Timing Control Register 1
	     */
		lcd32tim1_set.bit.FRMCNT = 0x01;		/// Frame frequency = 64.0 Hz.
		lcd32tim1_set.bit.COMLOC = 0;			/// COM pin layout = SEG64-79 COM31-16
		lcd32tim1_set.bit.LDUTY = 0x1F;			/// Drive duty = 1/32.
		LCD32TIM1 = lcd32tim1_set.reg;
	}

	/* LCD32TIM2:LCD32B Timing Control Register 2. */
	{
		/**
	     * @todo Change LCD32B Timing Control Register 2
	     */
		lcd32tim2_set.bit.BSTC = 1;				/// Booster clock frequency = fCLK_LCD32B/16 Hz.
		lcd32tim2_set.bit.NLINE = 0x00;			/// N-line inverse AC drive function = Normal drive.
		LCD32TIM2 = lcd32tim2_set.reg;
	}

	/* LCD32PWR:LCD32B Power Control Register. */
	{
		/**
	     * @todo Change LCD32B Power Control Register
	     */
		lcd32pwr_set.bit.EXVCSEL = 0;			/// LCD drive power supply mode = Internal generation mode.
		lcd32pwr_set.bit.LC	= 0xF;				/// LCD panel contrast = 0x00:light - 0xF:dark.
		lcd32pwr_set.bit.BSTEN = 1;				/// LCD voltage booster = off.
		lcd32pwr_set.bit.BIASSEL = 0;			/// LCD drive bias = 1/4(1)bias / 1/5(0)bias.
		lcd32pwr_set.bit.HVLD = 0;				/// LCD voltage regulator into heavy load protection mode = Normal mode.
		lcd32pwr_set.bit.VCEN = 1;				/// LCD voltage regulator = on.
		LCD32PWR = lcd32pwr_set.reg;
	}

	/* LCD32DSP:LCD32B Display Control Register. */
	{
		/**
	     * @todo Change LCD32B Display Control Register
	     */
		lcd32dsp_set.bit.SEGREV	= 1;			/// Segment pin assignment direction = Normal assignment.
		lcd32dsp_set.bit.COMREV	= 1;			/// Common pin assignment direction = Normal assignment.
		lcd32dsp_set.bit.DSPREV	= 1;			/// Black/White inversion = Normal display.
		lcd32dsp_set.bit.DSPAR = 0;				/// Display area in the display data RAM = Display area 0.
		lcd32dsp_set.bit.DSPC = 0;				/// LCD display = display off.
		LCD32DSP = lcd32dsp_set.reg;
	}

	/* LCD32COMC0,LCD32COMC1:LCD32B COM Pin Control Registers 0 and 1. */
	{
		/**
	     * @todo Change LCD32B COM Pin Control Registers 0 and 1
	     */
	 	lcd32comc0_set.bit.COM15DEN = 1;		/// Partial drive of COM15 = Normal output;
	 	lcd32comc0_set.bit.COM14DEN = 1;		/// Partial drive of COM14 = Normal output;
	 	lcd32comc0_set.bit.COM13DEN = 1;		/// Partial drive of COM13 = Normal output;
	 	lcd32comc0_set.bit.COM12DEN = 1;		/// Partial drive of COM12 = Normal output;
	 	lcd32comc0_set.bit.COM11DEN = 1;		/// Partial drive of COM11 = Normal output;
	 	lcd32comc0_set.bit.COM10DEN = 1;		/// Partial drive of COM10 = Normal output;
	 	lcd32comc0_set.bit.COM9DEN = 1;			/// Partial drive of COM9 = Normal output;
	 	lcd32comc0_set.bit.COM8DEN = 1;			/// Partial drive of COM8 = Normal output;
	 	lcd32comc0_set.bit.COM7DEN = 1;			/// Partial drive of COM7 = Normal output;
	 	lcd32comc0_set.bit.COM6DEN = 1;			/// Partial drive of COM6 = Normal output;
	 	lcd32comc0_set.bit.COM5DEN = 1;			/// Partial drive of COM5 = Normal output;
	 	lcd32comc0_set.bit.COM4DEN = 1;			/// Partial drive of COM4 = Normal output;
	 	lcd32comc0_set.bit.COM3DEN = 1;			/// Partial drive of COM3 = Normal output;
	 	lcd32comc0_set.bit.COM2DEN = 1;			/// Partial drive of COM2 = Normal output;
	 	lcd32comc0_set.bit.COM1DEN = 1;			/// Partial drive of COM1 = Normal output;
	 	lcd32comc0_set.bit.COM0DEN = 1;			/// Partial drive of COM0 = Normal output;
	 	LCD32COMC0 = lcd32comc0_set.reg;

	 	lcd32comc1_set.bit.COM31DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM30DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM29DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM28DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM27DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM26DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM25DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM24DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM23DEN = 1;		/// Partial drive of COM23 = Normal output;
	 	lcd32comc1_set.bit.COM22DEN = 1;		/// Partial drive of COM22 = Normal output;
	 	lcd32comc1_set.bit.COM21DEN = 1;		/// Partial drive of COM21 = Normal output;
	 	lcd32comc1_set.bit.COM20DEN = 1;		/// Partial drive of COM20 = Normal output;
	 	lcd32comc1_set.bit.COM19DEN = 1;		/// Partial drive of COM19 = Normal output;
	 	lcd32comc1_set.bit.COM18DEN = 1;		/// Partial drive of COM18 = Normal output;
	 	lcd32comc1_set.bit.COM17DEN = 1;		/// Partial drive of COM17 = Normal output;
	 	lcd32comc1_set.bit.COM16DEN = 1;		/// Partial drive of COM16 = Normal output;
	 	LCD32COMC1 = lcd32comc1_set.reg;
	}

	/// Clear interrupt flags.
	/* LCD32INTF:LCD32B Interrupt Flag Register. */
	{
		lcd32intf_set.bit.FRMIF = 1;
		LCD32INTF = lcd32intf_set.reg;
	}

	/// Disable LCD32B frame interrupts.
	/* LCD32INTE:LCD32B Interrupt Enable Register. */
	{
		lcd32inte_set.bit.FRMIE = 0;
		LCD32INTE = lcd32inte_set.reg;
	}

	/// initializes VRAM.
	c17clearLcd32bDisplayData(C17_LCD_DISPLAY_AREA0);
	c17clearLcd32bDisplayData(C17_LCD_DISPLAY_AREA1);

}


/************************************************************************//**
 * @brief   LCD32B Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intLcd32b(void)
{
 	LCD32INTF_REG	lcd32intf_set;

	/* LCD32INTF LCD32B Interrupt Flag Register read & clear */
	{
		lcd32intf_set.reg = LCD32INTF;
		LCD32INTF = lcd32intf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(lcd32intf_set.bit.FRMIF == 1)
	{
	   /**
		* @todo add interrupt function of frame interrupt.
		*/
	}

}
#endif

