/************************************************************************//**
 *
 * @file    c17_remc2.c
 *
 * @brief   This is REMC2 driver program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2017. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_remc2.h"
#include "c17_init_config.h"

#if defined(C17_PE_REMC)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13)
C17_INTERRUPT_HANDLER(_vector17_handler, c17intRemc);	// REMC interrupt.
#endif
#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18)
C17_INTERRUPT_HANDLER(_vector18_handler, c17intRemc);	// REMC interrupt.
#endif
#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
C17_INTERRUPT_HANDLER(_vector21_handler, c17intRemc);	// REMC interrupt.
#endif
#if defined(C17_MCUSEL_589)
C17_INTERRUPT_HANDLER(_vector23_handler, c17intRemc);	// REMC interrupt.
#endif
#endif /* CRT0_LINKED */


/* --- macro definition --- */
/// NEC format
/// 1T=562us
#define CRPER_NEC			(REMC_CLOCK * 1000000 / 38000 - 1)	//38KHz	  REMCCLK /38000000 -1
#define LEADER_APLEN_NEC	(562 * REMC_CLOCK * 16 - 1)		//16T	  0.000562(1T) * 16 * REMCCLK -1
#define LEADER_DBLEN_NEC    (562 * REMC_CLOCK * 24 - 1)		//16T+8T  0.000562(1T) * 24 * REMCCLK -1
#define BIT_0_APLEN_NEC		(562 * REMC_CLOCK * 1 - 1)		//1T      0.000562(1T) *  1 * REMCCLK -1
#define BIT_0_DBLEN_NEC	 	(562 * REMC_CLOCK * 2 - 1)		//1T+1T   0.000562(1T) *  2 * REMCCLK -1
#define BIT_1_APLEN_NEC		(562 * REMC_CLOCK * 1 - 1)		//1T      0.000562(1T) *  1 * REMCCLK -1
#define BIT_1_DBLEN_NEC	 	(562 * REMC_CLOCK * 4 - 1)		//1T+3T   0.000562(1T) *  4 * REMCCLK -1

// AEHA format
// 1T=425us
#define CRPER_AEHA			(REMC_CLOCK * 1000000 / 38000 - 1)	//38KHz	  REMCCLK /38000000 -1
#define LEADER_APLEN_AEHA	(425 * REMC_CLOCK * 8 - 1)		//8T      0.000425(1T) *  8 * REMCCLK -1
#define LEADER_DBLEN_AEHA   (425 * REMC_CLOCK * 12 - 1)		//8T+4T   0.000425(1T) * 12 * REMCCLK -1
#define BIT_0_APLEN_AEHA	(425 * REMC_CLOCK * 1 - 1)		//1T      0.000425(1T) *  1 * REMCCLK -1
#define BIT_0_DBLEN_AEHA	(425 * REMC_CLOCK * 2 - 1)		//1T+1T   0.000425(1T) *  2 * REMCCLK -1
#define BIT_1_APLEN_AEHA	(425 * REMC_CLOCK * 1 - 1)		//1T      0.000425(1T) *  1 * REMCCLK -1
#define BIT_1_DBLEN_AEHA	(425 * REMC_CLOCK * 4 - 1)		//1T+3T   0.000425(1T) *  4 * REMCCLK -1

// SONY format
// 1T=600us
#define CRPER_SONY			(REMC_CLOCK * 1000000 / 40000 - 1)	//40KHz	  REMCCLK /40000000 -1
#define LEADER_APLEN_SONY	(0)								//0
#define LEADER_DBLEN_SONY   (600 * REMC_CLOCK * 4 - 1)		//4T      0.000600(1T) * 4 * REMCCLK -1
#define BIT_0_APLEN_SONY	(600 * REMC_CLOCK * 1 - 1)		//1T      0.000600(1T) * 1 * REMCCLK -1
#define BIT_0_DBLEN_SONY	(600 * REMC_CLOCK * 2 - 1)		//1T+1T   0.000600(1T) * 2 * REMCCLK -1
#define BIT_1_APLEN_SONY	(600 * REMC_CLOCK * 1 - 1)		//1T      0.000600(1T) * 1 * REMCCLK -1
#define BIT_1_DBLEN_SONY	(600 * REMC_CLOCK * 3 - 1)		//1T+2T   0.000600(1T) * 3 * REMCCLK -1

/// REMC interrupt Status
enum
{
	STATUS_NONE,
	STATUS_DATA,
	STATUS_TR,
	STATUS_END
};

typedef struct st_remc_s {
	unsigned short	 Status;
	unsigned short	 bitCount;
	unsigned short sonybitCount;
	unsigned short sendEndFlg;
	unsigned short	 sendLength;
	unsigned char*	 sendData;
	unsigned char	 data;
} ST_REMC_S;

typedef struct st_remc_wave {
	unsigned short leader_aplen;		/// Leader APLEN
	unsigned short leader_dblen;		/// Leader DBLEN
	unsigned short bit1_aplen;			/// Data = 1 APLEN
	unsigned short bit1_dblen;			/// Data = 1 DBLEN
	unsigned short bit0_aplen;			/// Data = 0 APLEN
	unsigned short bit0_dblen;			/// Data = 0 DBLEN
} ST_REMC_WAVE;


/* --- static --- */
static unsigned char  send_format;
static ST_REMC_S		remc_send;
static ST_REMC_WAVE		remc_wave;

static void necFormatInt(void);
static void sonyFormatInt(void);

/************************************************************************//**
 * @brief   Initialize Remc.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initRemc(void)
{
	REMCLK_REG		remclk_set;
	REMDBCTL_REG	remdbctl_set;

#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13)
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;
#endif
#if defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16)
	P1MODSEL_REG	p1modsel_set;
	P1FNCSEL_REG	p1fncsel_set;
#endif
#if defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W18)
	P2MODSEL_REG	p2modsel_set;
	P2FNCSEL_REG	p2fncsel_set;
#endif
#if defined(C17_MCUSEL_589)
	P3MODSEL_REG	p3modsel_set;
	P3FNCSEL_REG	p3fncsel_set;
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13)
	P5MODSEL_REG	p5modsel_set;
	P5FNCSEL_REG	p5fncsel_set;
#endif
#if defined(C17_MCUSEL_W18)
	P6MODSEL_REG	p6modsel_set;
	P6FNCSEL_REG	p6fncsel_set;
#endif

	// REMCRST=1  Reset REMC.
	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.REMCRST = 1;
		REMDBCTL = remdbctl_set.reg;
	}

	/* REMCLK:REMC2 Clock Control Register */
	{
		remclk_set.bit.DBRUN = 0;
		remclk_set.bit.CLKDIV = (REMC_SET_INIT & 0x0F);
		remclk_set.bit.CLKSRC = (REMC_SET_INIT & 0xC0)>>6;
		REMCLK = remclk_set.reg;
	}

	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.bit.PRESET = 0;
		remdbctl_set.bit.PRUN =0;
		remdbctl_set.bit.REMOINV = 0;
		remdbctl_set.bit.BUFEN = 0;
		remdbctl_set.bit.TRMD = (REMC_SET_INIT & 0x20)>>5;
		remdbctl_set.bit.REMCRST = 0;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;
	}

#if defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P00 : REMO
	 *  P01 : CLPLS
	 */
	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P01SEL = 1;		/// - P01 : Enable peripheral I/O function.
		p0modsel_set.bit.P00SEL = 1;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P01MUX = 0;		/// - P01 : CLPLS.
		p0fncsel_set.bit.P00MUX = 0;		/// - P00 : REMO.
		P0FNCSEL = p0fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P16 : REMO
	 *  P17 : CLPLS
	 */
	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P17SEL = 1;		/// - P17 : Enable peripheral I/O function.
		p1modsel_set.bit.P16SEL = 1;		/// - P16 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;
	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P17MUX = 0;		/// - P17 : CLPLS.
		p1fncsel_set.bit.P16MUX = 0;		/// - P16 : REMO.
		P1FNCSEL = p1fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P20 : REMO
	 *  P21 : CLPLS
	 */
	/* P2MODSEL:P2 Port Mode Select Register */
	{
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P21SEL = 1;		/// - P21 : Enable peripheral I/O function.
		p2modsel_set.bit.P20SEL = 1;		/// - P20 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;
	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P21MUX = 0;		/// - P51 : CLPLS.
		p2fncsel_set.bit.P20MUX = 0;		/// - P50 : REMO.
		P2FNCSEL = p2fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P50 : REMO
	 *  P51 : CLPLS
	 */
	/* P5MODSEL:P5 Port Mode Select Register */
	{
		p5modsel_set.reg = P5MODSEL;
		p5modsel_set.bit.P51SEL = 1;		/// - P51 : Enable peripheral I/O function.
		p5modsel_set.bit.P50SEL = 1;		/// - P50 : Enable peripheral I/O function.
		P5MODSEL = p5modsel_set.reg;
	}
	/* P5FNCSEL: P5 Port Function Select Register */
	{
		p5fncsel_set.reg = P5FNCSEL;
		p5fncsel_set.bit.P51MUX = 0;		/// - P51 : CLPLS.
		p5fncsel_set.bit.P50MUX = 0;		/// - P50 : REMO.
		P5FNCSEL = p5fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_W18)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P62 : REMO
	 *  P26 : CLPLS
	 */
	/* P2MODSEL:P2 Port Mode Select Register */
	{
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P26SEL = 1;		/// - P26 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;
	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P26MUX = 0;		/// - P26 : CLPLS.
		P2FNCSEL = p2fncsel_set.reg;
	}
	/* P6MODSEL:P6 Port Mode Select Register */
	{
		p6modsel_set.reg = P6MODSEL;
		p6modsel_set.bit.P62SEL = 1;		/// - P62 : Enable peripheral I/O function.
		P6MODSEL = p6modsel_set.reg;
	}
	/* P6FNCSEL: P6 Port Function Select Register */
	{
		p6fncsel_set.reg = P6FNCSEL;
		p6fncsel_set.bit.P62MUX = 0;		/// - P62 : REMO.
		P6FNCSEL = p6fncsel_set.reg;
	}
#endif
#if defined(C17_MCUSEL_589)
	/// Initialize the REMC2 output function to the ports.
	/*
	 *  P34 : REMO
	 *  P36 : CLPLS
	 */
	/* P3MODSEL:P3 Port Mode Select Register */
	{
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P36SEL = 1;		/// - P36 : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = 1;		/// - P34 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;
	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P36MUX = 0;		/// - P36 : CLPLS.
		p3fncsel_set.bit.P34MUX = 0;		/// - P34 : REMO.
		P3FNCSEL = p3fncsel_set.reg;
	}
#endif

	/// Initialize the interrupt.
	ILVREMC_0 = INTERRUPT_LEVEL_REMC;
}


/************************************************************************//**
 * @brief   Remc enable.
 *
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc disable.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.PRUN = 0;
		REMDBCTL = remdbctl_set.reg;

		remdbctl_set.bit.MODEN = 0;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   REMC Interrupt enable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntRemc(void)
{
	REMINTF_REG		remintf_set;
	REMINTE_REG		reminte_set;

	//Clear interrupt flag.
	/* REMINTF: REMC2 Status and Interrupt Flag Register. */
	{
		remintf_set.reg = 0;
		remintf_set.bit.DBIF = 1;
		remintf_set.bit.APIF = 1;
		REMINTF	= remintf_set.reg;
	}

	/**
	 * @todo Change interrupt
	 */
	// DBIF=1 APIF=0  Enable interrupt.
	/* REMINTE: REMC2 Interrupt Enable Register. */
	{
		reminte_set.reg = 0;
		reminte_set.bit.DBIE = 1;
		reminte_set.bit.APIE = 0;
		REMINTE = reminte_set.reg;
	}
}


/************************************************************************//**
 * @brief   REMC Interrupt disable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntRemc(void)
{
	REMINTF_REG		remintf_set;
	REMINTE_REG		reminte_set;

	//Clear interrupt flag.
	/* REMINTF: REMC2 Status and Interrupt Flag Register. */
	{
		remintf_set.reg = 0;
		remintf_set.bit.DBIF = 1;
		remintf_set.bit.APIF = 1;
		REMINTF	= remintf_set.reg;
	}

	// DBIF=0 APIF=0  Disable interrupt.
	/* REMINTE: REMC2 Interrupt Enable Register. */
	{
		reminte_set.reg = 0;
		reminte_set.bit.DBIE = 0;
		reminte_set.bit.APIE = 0;
		REMINTE = reminte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set Carrier Remc.
 *
 * @param   remcCarr:Carrier Waveform
 *  @li @c   0 to 255
 * @param   remcDuty:DUTY
 *  @li @c   0 to 255
 *
 * @retval  none
 ***************************************************************************/
void c17setCarrierRemc(unsigned short remcCarr, unsigned short remcDuty)
{
	REMCCTL_REG		remcctl_set;
	REMCARR_REG		remcarr_set;

	/* REMCARR: REMC2 Carrier Waveform Register. */
	{
		remcarr_set.bit.CRDTY = remcDuty;
		remcarr_set.bit.CRPER = remcCarr;
		REMCARR		= remcarr_set.reg;					///	Set carrier waveform.
	}

	/// Enable carrier modulation.
	/* REMCCTL: REMC2 Carrier Modulation Control Register. */
	{
		remcctl_set.reg = 0;
		remcctl_set.bit.CARREN = 1;
		REMCCTL = remcctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc start.
 *
 * @param   remcAplen      data bit active pulse length.
 *  @li @c   0 to 65535
 * @param   remcDblen      data bit length.
 *  @li @c   0 to 65535
 *
 * @retval  none
 ***************************************************************************/
void c17startRemc(unsigned short remcAplen, unsigned short remcDblen)
{
	REMDBCTL_REG	remdbctl_set;

	REMAPLEN = remcAplen;			/// Set data bit active pulse length.
	REMDBLEN = remcDblen;			/// Set data bit length.

	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;

		remdbctl_set.bit.PRESET = 1;
		remdbctl_set.bit.PRUN = 1;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc stop.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.PRUN = 0;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Send Remc data.
 *
 * @param   remcData:   Send data pointer
 * @param   remcLen:    Send data length
 *
 * @retval  none
 ***************************************************************************/
void c17sendRemc(const unsigned char* remcData, unsigned char remcLen)
{
	remc_send.sendData = (unsigned char*)remcData;
	remc_send.sendLength = remcLen;
	remc_send.Status = STATUS_DATA;
	remc_send.sendEndFlg = 0;
	remc_send.bitCount = 0;

	c17startRemc(remc_wave.leader_aplen,remc_wave.leader_dblen);
}


/************************************************************************//**
 * @brief   REMC transmit end wait.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17sendRemcDataWait(void)
{
	if(remc_send.sendEndFlg == 1)
	{
		return EXIT_SUCCESS;
	}
	else
	{
		return EXIT_FAILURE;
	}
}


/************************************************************************//**
 * @brief   1, Set Carrier Remc.
 *
 *          2, Set Data Bit Length and,Data Bit Active Pulse Length.
 *
 * @param   format
 *
 * @retval  none
 ***************************************************************************/
void c17setFormatRemc(unsigned short remcFormat)
{
	REMDBCTL_REG	remdbctl_set;

	send_format = remcFormat;

	c17disableIntRemc();
	switch(send_format)
	{
		case C17_REMC_FORMAT_NEC:
			c17setCarrierRemc(CRPER_NEC, CRPER_NEC/3);		// Set carrier waveform for NEC format
															// 38kHz, 1/3duty
															// CRPER = REMC_CLOCK/38KHz -1 = CRPER_NEC
															// CRDTY = CRPER_NEC/3

			/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 0;
				REMDBCTL = remdbctl_set.reg;
			}

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_NEC;
			remc_wave.leader_dblen = LEADER_DBLEN_NEC;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_NEC;
			remc_wave.bit1_dblen = BIT_1_DBLEN_NEC;
			remc_wave.bit0_aplen = BIT_0_APLEN_NEC;
			remc_wave.bit0_dblen = BIT_0_DBLEN_NEC;
			break;
		case C17_REMC_FORMAT_AEHA:
			c17setCarrierRemc(CRPER_AEHA, CRPER_AEHA/3);	// Set carrier waveform for AEHA format
															// 38kHz, 1/3duty
															// CRPER = REMC_CLOCK/38KHz -1 = CRPER_AEHA
															// CRDTY = CRPER_AEHA/3
			/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 0;
				REMDBCTL = remdbctl_set.reg;
			}

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_AEHA;
			remc_wave.leader_dblen = LEADER_DBLEN_AEHA;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_AEHA;
			remc_wave.bit1_dblen = BIT_1_DBLEN_AEHA;
			remc_wave.bit0_aplen = BIT_0_APLEN_AEHA;
			remc_wave.bit0_dblen = BIT_0_DBLEN_AEHA;
			break;
		case C17_REMC_FORMAT_SONY:
			c17setCarrierRemc(CRPER_SONY, CRPER_SONY/3);	// Set carrier waveform for SONY format
															// 40kHz, 1/3duty
															// CRPER = REMC_CLOCK/40KHz -1 = CRPER_SONY
															// CRDTY = CRPER_SONY/3

			/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 1;
				REMDBCTL = remdbctl_set.reg;
			}

			remc_send.sonybitCount = 0;

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_SONY;
			remc_wave.leader_dblen = LEADER_DBLEN_SONY;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_SONY;
			remc_wave.bit1_dblen = BIT_1_DBLEN_SONY;
			remc_wave.bit0_aplen = BIT_0_APLEN_SONY;
			remc_wave.bit0_dblen = BIT_0_DBLEN_SONY;
			break;
	}

	c17enableIntRemc();
}


/************************************************************************//**
 * @brief   Remc interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intRemc(void)
{
	REMINTF_REG	remintf_set;

	/* Capture interrupt flag */
	remintf_set.reg = REMINTF;
	/* Clear Interrupt flag. */
	REMINTF = remintf_set.reg;

	/// Check interrupt cause occurrence status.
	if (remintf_set.bit.APIF == 1) {

	   /**
		* @todo add interrupt function of APIF
		*/
	}
	/// Check interrupt cause occurrence status.
	if (remintf_set.bit.DBIF == 1) {
	   /**
		* @todo add interrupt function of DBIF
		*/
		if(send_format == C17_REMC_FORMAT_NEC || send_format == C17_REMC_FORMAT_AEHA)
		{
			necFormatInt();
		}
		else if(send_format == C17_REMC_FORMAT_SONY)
		{
			sonyFormatInt();
		}
	}
}


/************************************************************************//**
 * @brief   For NEC format & AEHA format interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void necFormatInt(void)
{
	switch(remc_send.Status)
	{
		case STATUS_NONE:
			break;
		case STATUS_DATA:
			if(remc_send.bitCount == 0){
				remc_send.data = *remc_send.sendData++;
				remc_send.sendLength--;
			}
			if(remc_send.data & 0x80){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}
			remc_send.data <<= 1;
			remc_send.bitCount++;
			if(remc_send.bitCount == 8){
				remc_send.bitCount = 0;

				if(remc_send.sendLength == 0){
					remc_send.Status = STATUS_TR;
					break;
				}
			}
			break;
		case STATUS_TR:
			c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			remc_send.Status = STATUS_END;
			break;
		case STATUS_END:
			remc_send.sendEndFlg = 1;
			remc_send.Status = STATUS_NONE;
			c17stopRemc();
			break;
	}
}


/************************************************************************//**
 * @brief   For SONY format interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void sonyFormatInt(void)
{
	switch(remc_send.Status)
	{
		case STATUS_NONE:
			break;
		case STATUS_DATA:
			if(remc_send.bitCount==0){
				remc_send.data = *remc_send.sendData++;
				remc_send.sendLength--;
			}

			if(remc_send.data & 0x80){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}
			remc_send.data <<= 1;
			remc_send.bitCount++;
			remc_send.sonybitCount++;
			if(remc_send.bitCount == 8){
				remc_send.bitCount = 0;
			}
			if(remc_send.sonybitCount == 12){
				remc_send.Status = STATUS_TR;
				break;
			}
			break;
		case STATUS_TR:
			remc_send.sendEndFlg = 1;
			remc_send.Status = STATUS_NONE;
			break;
	}
}

#endif

