/************************************************************************//**
 *
 * @file    c17_t16b.c
 *
 * @brief   This is T16 PWM Timer(T16B) program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2017. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_t16b.h"
#include "c17_init_config.h"

#if defined(C17_PE_T16B)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector14_handler, c17intT16bCh0);		// T16B Ch.0 interrupt
#endif /* CRT0_LINKED */


/* --- macro definition --- */
#define CH_MAX_COUNT	1

/* --- static const variable --- */
volatile T16BNCLK_REG* const t16bnclk_table[CH_MAX_COUNT] = {(T16BNCLK_REG *) & T16B0CLK};
volatile T16BNCTL_REG* const t16bnctl_table[CH_MAX_COUNT] = {(T16BNCTL_REG *) & T16B0CTL};
volatile T16BNMC_REG* const t16bnmc_table[CH_MAX_COUNT] = {(T16BNMC_REG *) & T16B0MC};
volatile T16BNCCCTLM_REG* const t16bnccctl0_table[CH_MAX_COUNT] = {(T16BNCCCTLM_REG *) & T16B0CCCTL0};
volatile T16BNCCCTLM_REG* const t16bnccctl1_table[CH_MAX_COUNT] = {(T16BNCCCTLM_REG *) & T16B0CCCTL1};
volatile T16BNCCRM_REG* const t16bnccr0_table[CH_MAX_COUNT] = {(T16BNCCRM_REG *) & T16B0CCR0};
volatile T16BNCCRM_REG* const t16bnccr1_table[CH_MAX_COUNT] = {(T16BNCCRM_REG *) & T16B0CCR1};
volatile T16BNINTF_REG* const t16bnintf_table[CH_MAX_COUNT] = {(T16BNINTF_REG *) & T16B0INTF};
volatile T16BNINTE_REG* const t16bninte_table[CH_MAX_COUNT] = {(T16BNINTE_REG *) & T16B0INTE};

/* --- global variable --- */
// Interrupt counter.
/**
 * @todo t16bCh0IntComparator = Interruption counter.
 *       t16bCh0IntCapture    = Interruption counter.
 *       t16bCh0IntMax        = Interruption counter.
 *       t16bCh0IntZero       = Interruption counter.
 */
volatile unsigned short t16bCh0IntComparator	= 0;	///< Comparator interrupt counter.
volatile unsigned short t16bCh0IntCapture	= 0;	///< Compare interrupt counter.
volatile unsigned short t16bCh0IntMax		= 0;	///< Counter MAX interrupt counter.
volatile unsigned short t16bCh0IntZero		= 0;	///< Counter zero interrupt counter.

/// Compare interrupt counter data.
volatile unsigned short t16bCh0IntCaptureData[ T16B_CH0_INT_CAP_DATA_MAX ];
/// Compare interrupt counter data index.
volatile unsigned int t16bCh0IntCaptureIndex;

void setTOUT(void);

/************************************************************************//**
 * @brief   Initialization of the T16B Register.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initT16b(void)
{
	/// T16B CH 0 Setting
	if((T16BCH0_INIT & 0x0400) == 0x0400)
	{
		/// Initialize T16 timer.
		c17setT16b(0, (T16BCH0_INIT & 0x0380)>>7, (T16BCH0_INIT & 0x000F), (T16BCH0_INIT & 0x0060)>>5, (T16BCH0_INIT & 0x0010)>>4);
	}
}

/************************************************************************//**
 * @brief   T16B timer set.
 *
 * @param   t16BChannel      Timer CH.
 *  @li @c  0 to CH_MAX_COUNT-1
 * @param   t16BClkSrc       Clock Source.
 *  @li @c  C17_T16B_CLKSRC_IOSC
 *  @li @c  C17_T16B_CLKSRC_OSC1
 *  @li @c  C17_T16B_CLKSRC_OSC3
 *  @li @c  C17_T16B_CLKSRC_EXT
 *  @li @c  C17_T16B_CLKSRC_EXCL0
 *  @li @c  C17_T16B_CLKSRC_EXCL1
 *  @li @c  C17_T16B_CLKSRC_EXCL0INV
 *  @li @c  C17_T16B_CLKSRC_EXCL1INV
 * @param   t16BClkDiv       Clock Division
 *  @li @c  C17_T16B_CLKDIV_1 to C17_T16B_CLKDIV_32768
 * @param   t16BUpDown       Mode
 *  @li @c  C17_T16B_UP      Up count.
 *  @li @c  C17_T16B_DOWN    Down count.
 *  @li @c  C17_T16B_UP_DOWN Up/Down count.
 * @param   t16BMode         Mode
 *  @li @c  C17_T16B_ONESHOT One shot.
 *  @li @c  C17_T16B_REPEAT  Repeat.
 *
 * @retval  none
 ***************************************************************************/
void c17setT16b(unsigned short t16BChannel, unsigned short t16BClkSrc, unsigned short t16BClkDiv, unsigned short t16BUpDown, unsigned short t16BMode)
{
	T16BNCLK_REG	t16bnclk_set;
	T16BNCTL_REG	t16bnctl_set;
	T16BNMC_REG		t16bnmc_set;
	T16BNCCCTLM_REG	t16bnccctl0_set;
	T16BNCCCTLM_REG	t16bnccctl1_set;
	T16BNCCRM_REG	t16bnccr0_set;
	T16BNCCRM_REG	t16bnccr1_set;
	T16BNINTF_REG	t16bnintf_set;
	T16BNINTE_REG	t16bninte_set;

	if(t16BChannel >= CH_MAX_COUNT) {
		return;
	}

	/// Stop T16b Ch.N Timer.
	c17stopT16b(t16BChannel);

	/// Set Enable T16B operations.
	/* T16BNCTL:T16B Ch.N Counter Control Register. */
	{
		t16bnctl_set.reg = 0;
		t16bnctl_set.bit.CNTMD = t16BUpDown;	/// - Set count up operation.
		t16bnctl_set.bit.ONEST = t16BMode;		/// - Set repeat operation.
		t16bnctl_set.bit.RUN = 0;				/// - Stops counting.
		t16bnctl_set.bit.PRESET = 0;
		t16bnctl_set.bit.MODEN = 1;				/// - Enables the T16B Ch.N operations.
		t16bnctl_table[t16BChannel]->reg = t16bnctl_set.reg;
	}

	/// Set clock source for T16b Ch.N.
	/* T16BNCLK:T16B Ch.N Clock Control Register. */
	{
		t16bnclk_set.reg = 0;
		t16bnclk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode;
		if(t16BClkSrc == C17_T16B_CLKSRC_OSC1)
		{
			/**
			 *  Clock source = OSC1,Clock division = C17_T16B_CLKDIV_1 to C17_T16B_CLKDIV_256
			 */
			if(t16BClkDiv > C17_T16B_CLKDIV_256)
			{
				t16BClkDiv = C17_T16B_CLKDIV_256;
			}
		}
		t16bnclk_set.bit.CLKDIV = t16BClkDiv;	/// - Set division ratio for clock source.;
		t16bnclk_set.bit.CLKSRC = t16BClkSrc;	/// - Set clock source.
		t16bnclk_table[t16BChannel]->reg = t16bnclk_set.reg;
	}

	/// Set comparator/capture circuit 0 to comparator mode.
	/* T16BNCCCTL0:T16B Ch.N Comparator/Capture 0 Control Register. */
	{
		/**
		 * @todo Change T16BNCCCTL0
		 */
		t16bnccctl0_set.reg = 0;
		t16bnccctl0_set.bit.SCS = 0;		/// - Either capture mode(0=Asynchronous/1=Synchronous).
		t16bnccctl0_set.bit.CBUFMD = 0;		/// - Set Immediately after writing to the T16BNCCR0 register.
		t16bnccctl0_set.bit.CAPIS = 0;		/// - Set External trigger signal.
		t16bnccctl0_set.bit.CAPTRG = 0;		/// - Set Not triggered.
		t16bnccctl0_set.bit.TOUTMT = 0;		/// - Set waveform generation signal(MAX/zero signals).
		t16bnccctl0_set.bit.TOUTO = 0;		/// - Set signal output level to Low.
		t16bnccctl0_set.bit.TOUTMD = 3;		/// - Set signal waveform to set/reset mode.
		t16bnccctl0_set.bit.TOUTINV = 0;	/// - TOUT00 signal polarity = Active High
		t16bnccctl0_set.bit.CCMD = 0;		/// - The operating mode(0=Comparator/1=Capture).
		t16bnccctl0_table[t16BChannel]->reg = t16bnccctl0_set.reg;
	}

	/// Set comparator/capture circuit 1 to capture mode.
	/* T16BNCCCTL1:T16B Ch.N Comparator/Capture 1 Control Register. */
	{
		/**
		 * @todo Change T16BNCCCTL1
		 */
		t16bnccctl1_set.reg = 0;
		t16bnccctl1_set.bit.SCS = 1;		/// - Either capture mode(0=Asynchronous/1=Synchronous).
		t16bnccctl1_set.bit.CBUFMD = 0;		/// - Set Immediately after writing to the T16B0CCR0 register.
		t16bnccctl1_set.bit.CAPIS = 2;		/// - Set Software trigger signal(LOW).
		t16bnccctl1_set.bit.CAPTRG = 3;		/// - Set Trigger edge(LOW->HIGH or HIGH->LOW).
		t16bnccctl1_set.bit.TOUTMT = 0;		/// - Set waveform generation signal(MAX/zero signals).
		t16bnccctl1_set.bit.TOUTO = 0;		/// - Set signal output level to Low.
		t16bnccctl1_set.bit.TOUTMD = 0;		/// - Set signal waveform to set/reset mode.
		t16bnccctl1_set.bit.TOUTINV = 0;	/// - TOUT00 signal polarity = Active High
		t16bnccctl1_set.bit.CCMD = 1;		/// - The operating mode(0=Comparator/1=Capture).
		t16bnccctl1_table[t16BChannel]->reg = t16bnccctl1_set.reg;
	}

	/// Set the counter comparison value.
	/* T16BNCCR0:T16B Ch.N Compare/Capture 0 Data Register. */
	/* T16BNCCR1:T16B Ch.N Compare/Capture 1 Data Register. */
	{
		t16bnccr0_set.reg = 0xFFFF;
		t16bnccr0_table[t16BChannel]->reg = t16bnccr0_set.reg;

		t16bnccr1_set.reg = 0xFFFF;
		t16bnccr1_table[t16BChannel]->reg = t16bnccr1_set.reg;

	}

	/// Set MAX counter data.
	/* T16BNMC:T16B Ch.N Max Counter Data Register */
	{
		t16bnmc_set.reg = 0xFFFF;
		t16bnmc_table[t16BChannel]->reg = t16bnmc_set.reg;
	}

	// Clear interrupt flags.
	/* T16BNINTF:T16B Ch.N Interrupt Flag Register. */
	{
		t16bnintf_set.reg = 0;
  		t16bnintf_set.bit.CAPOW5IF  = 0;
  		t16bnintf_set.bit.CMPCAP5IF = 0;
		t16bnintf_set.bit.CAPOW4IF  = 0;
		t16bnintf_set.bit.CMPCAP4IF = 0;
		t16bnintf_set.bit.CAPOW3IF  = 0;
		t16bnintf_set.bit.CMPCAP3IF = 0;
		t16bnintf_set.bit.CAPOW2IF  = 0;
		t16bnintf_set.bit.CMPCAP2IF = 0;
		t16bnintf_set.bit.CAPOW1IF  = 1;
		t16bnintf_set.bit.CMPCAP1IF = 1;
		t16bnintf_set.bit.CAPOW0IF  = 1;
		t16bnintf_set.bit.CMPCAP0IF = 1;
		t16bnintf_set.bit.CNTMAXIF  = 1;
		t16bnintf_set.bit.CNTZEROIF = 1;
  		t16bnintf_table[t16BChannel]->reg = t16bnintf_set.reg;
	}

	// Disable interrupts.
	/* T16BNINTE:T16B Ch.N Interrupt Enable Register. */
	{
		t16bninte_set.reg = 0;
  		t16bninte_set.bit.CAPOW5IE  = 0;
  		t16bninte_set.bit.CMPCAP5IE = 0;
		t16bninte_set.bit.CAPOW4IE  = 0;
		t16bninte_set.bit.CMPCAP4IE = 0;
		t16bninte_set.bit.CAPOW3IE  = 0;
		t16bninte_set.bit.CMPCAP3IE = 0;
		t16bninte_set.bit.CAPOW2IE  = 0;
		t16bninte_set.bit.CMPCAP2IE = 0;
		t16bninte_set.bit.CAPOW1IE  = 0;
		t16bninte_set.bit.CMPCAP1IE = 0;
		t16bninte_set.bit.CAPOW0IE  = 0;
		t16bninte_set.bit.CMPCAP0IE = 0;
		t16bninte_set.bit.CNTMAXIE  = 0;
  		t16bninte_set.bit.CNTZEROIE = 0;
  		t16bninte_table[t16BChannel]->reg = t16bninte_set.reg;
	}

	// Reset counter(global variable).
	c17resetT16bCh0IntCount();

	/// Initialize the interrupt.
	ILVT16B_0 = INTERRUPT_LEVEL_T16B_0;

	// TOUT0=P00
	setTOUT();

}


/************************************************************************//**
 * @brief   Set TOUT.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void setTOUT(void)
{
	/**
	 * @todo P00=TOUT0 Change port
	 */
	P0UPMUX0_REG	p0upmux_set;
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P00SEL = 1;		/// - P00 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}

	/* P0FNCSEL:P0 Port Function Select Register. */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P00MUX = 1;		/// - P00 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/* P0UPMUX0:P00-01 Universal Port Multiplexer Setting Register */
	{
		p0upmux_set.reg = P0UPMUX0;
		p0upmux_set.bit.P00PPFNC = 1;		/// - P00 : TOUT0
		p0upmux_set.bit.P00PERICH = 0;		/// - P00 : CH.0
		p0upmux_set.bit.P00PERISEL = 4;		/// - P00 : T16B.
		P0UPMUX0 = p0upmux_set.reg;
	}
}


/************************************************************************//**
 * @brief   Start T16b.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startT16b(unsigned short t16BChannel)
{
	T16BNCTL_REG	t16bnctl_set;
	T16BNINTF_REG	t16bnintf_set;
	T16BNINTE_REG	t16bninte_set;

	if(t16BChannel >= CH_MAX_COUNT) {
		return;
	}

	// Clear interrupt flags.
	//T16B0INTF_CNTZEROIF	= 1;		/// - Counter zero interrupt flag.
	//T16B0INTF_CNTMAXIF	= 1;		/// - Counter MAX interrupt flag.
	//T16B0INTF_CMPCAP0IF	= 1;		/// - Compare/capture 0 interrupt flag.
	//T16B0INTF_CAPOW0IF	= 1;		/// - Capture 0 overwrite interrupt flag.
	//T16B0INTF_CMPCAP1IF	= 1;		/// - Compare/capture 1 interrupt flag.
	//T16B0INTF_CAPOW1IF	= 1;		/// - Capture 1 overwrite interrupt flag.
	/* T16BNINTF:T16B Ch.N Interrupt Flag Register */
	{
		t16bnintf_set.reg = 0x003F;
		t16bnintf_table[t16BChannel]->reg = t16bnintf_set.reg;		///- Clear interrupt flag.
	}

	/**
	 * @todo Change Enable interrupts.
	 */
	/* T16BNINTE:T16B Ch.N Interrupt Enable Register. */
	{
		t16bninte_set.reg = 0;
  		t16bninte_set.bit.CAPOW5IE  = 0;
  		t16bninte_set.bit.CMPCAP5IE = 0;
		t16bninte_set.bit.CAPOW4IE  = 0;
		t16bninte_set.bit.CMPCAP4IE = 0;
		t16bninte_set.bit.CAPOW3IE  = 0;
		t16bninte_set.bit.CMPCAP3IE = 0;
		t16bninte_set.bit.CAPOW2IE  = 0;
		t16bninte_set.bit.CMPCAP2IE = 0;

		t16bninte_set.bit.CAPOW1IE  = 1;	/// - Capture 1 overwrite interrupt.
		t16bninte_set.bit.CMPCAP1IE = 1;	/// - Compare/capture 1 interrupt.
		t16bninte_set.bit.CAPOW0IE  = 0;	/// - Capture 0 overwrite interrupt.
		t16bninte_set.bit.CMPCAP0IE = 1;	/// - Compare/capture 0 interrupt.
		t16bninte_set.bit.CNTMAXIE  = 1;	/// - Counter MAX interrupt.
  		t16bninte_set.bit.CNTZEROIE = 1;	/// - Counter zero interrupt.
  		t16bninte_table[t16BChannel]->reg = t16bninte_set.reg;
	}

	/// Set Enable T16B operations.
	/* T16BNCTL:T16B Ch.N Counter Control Register. */
	{
		t16bnctl_set.reg = t16bnctl_table[t16BChannel]->reg;
		t16bnctl_set.bit.PRESET = 1;		/// Reset counter.
		t16bnctl_table[t16BChannel]->reg = t16bnctl_set.reg;

		t16bnctl_set.bit.RUN = 1;			/// Start counting.
		t16bnctl_set.bit.PRESET = 0;
		t16bnctl_table[t16BChannel]->reg = t16bnctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Stop T16b.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopT16b(unsigned short t16BChannel)
{
	T16BNCTL_REG	t16bnctl_set;

	if(t16BChannel >= CH_MAX_COUNT) {
		return;
	}

	/// Stop counting.
	/* T16BNCTL:T16B Ch.N Counter Control Register. */
	{
		t16bnctl_set.reg = t16bnctl_table[t16BChannel]->reg;
		t16bnctl_set.bit.RUN = 0;		/// Reset counter.
		t16bnctl_table[t16BChannel]->reg = t16bnctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Get comparator count.
 *
 * @param   none
 *
 * @retval  Comparator count.
 ***************************************************************************/
unsigned short c17getT16bCh0ComparatorCount(void)
{
	/// Get comparator count.
	return ((unsigned short)T16B0CCR0);
}


/************************************************************************//**
 * @brief   Set comparator count.
 *
 * @param   count  Comparator count.
 *
 * @retval  none
 ***************************************************************************/
void c17setT16bCh0ComparatorCount(unsigned short count)
{
	/// Set comparator count.
	T16B0CCR0 = count;
}


/************************************************************************//**
 * @brief   Get timer count.
 *
 * @param   none.
 *
 * @retval  Timer count.
 ***************************************************************************/
unsigned short c17getT16bCh0TimerCount(void)
{
	/// Get comparator count.
	return ((unsigned short)T16B0TC);
}


/************************************************************************//**
 * @brief   Get max count.
 *
 * @param   none.
 *
 * @retval  Max count.
 ***************************************************************************/
unsigned short c17getT16bCh0MaxCount(void)
{
	/// Get comparator count.
	return ((unsigned short)T16B0MC);
}


/************************************************************************//**
 * @brief   Get max count.
 *
 * @param   Set max count.
 *
 * @retval  none
 ***************************************************************************/
void c17setT16bCh0MaxCount(unsigned short count)
{
	/// Set comparator count.
	T16B0MC	= count;
}


/************************************************************************//**
 * @brief   Reset interrupt counter.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17resetT16bCh0IntCount(void)
{
	unsigned int i;

	/// Reset counter
	t16bCh0IntComparator	= 0;	// Comparator interrupt count.
	t16bCh0IntCapture		= 0;	// Compare interrupt count.
	t16bCh0IntMax			= 0;	// Counter MAX interrupt count.
	t16bCh0IntZero			= 0;	// Counter zero interrupt count.

	/// Reset compare interrupt counter data.
	t16bCh0IntCaptureIndex	= 0;
	for (i = 0; i < T16B_CH0_INT_CAP_DATA_MAX; i++) {
		t16bCh0IntCaptureData[i] = 0;
	}
}


/************************************************************************//**
 * @brief   Get counter MAX interrupt count.
 *
 * @param   none
 *
 * @retval  Counter MAX interrupt count.
 ***************************************************************************/
unsigned short c17getT16bCh0IntMaxCount(void)
{
	return (t16bCh0IntMax);
}


/************************************************************************//**
 * @brief   Get counter zero interrupt count.
 *
 * @param   none
 *
 * @retval  Counter zero interrupt count.
 ***************************************************************************/
unsigned short c17getT16bCh0IntZeroCount(void)
{
	return (t16bCh0IntZero);
}


/************************************************************************//**
 * @brief   Get compare interrupt count.
 *
 * @param   none
 *
 * @retval  Compare interrupt count.
 ***************************************************************************/
unsigned short c17getT16bCh0IntCaptureCount(void)
{
	return (t16bCh0IntCapture);
}


/************************************************************************//**
 * @brief   Get count value when compare interrupt.
 *
 * @param   index   Compare interrupt index.
 *
 * @retval  Count value when compare interrupt.
 ***************************************************************************/
unsigned short c17getT16bCh0IntCaptureCountData(unsigned int index)
{
	if (index >= T16B_CH0_INT_CAP_DATA_MAX) {
		return 0;
	}

	return (t16bCh0IntCaptureData[index]);
}


/************************************************************************//**
 * @brief   Get comparator interrupt count.
 *
 * @param   none
 *
 * @retval  Comparator interrupt count.
 ***************************************************************************/
unsigned short c17getT16bCh0IntComparatorCount(void)
{
	return (t16bCh0IntComparator);
}


/************************************************************************//**
 * @brief   T16B(Ch.0) interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intT16bCh0(void)
{
	T16BNINTF_REG	t16b0intf_set;
	T16BNCCCTLM_REG	t16b0ccctl1_set;

	/* Capture interrupt flag */
	t16b0intf_set.reg = T16B0INTF;

	/* Clear T16B Ch.0 Interrupt flag. */
	T16B0INTF = t16b0intf_set.reg;

	/// Check counter zero interrupt flag.
	if(t16b0intf_set.bit.CNTZEROIF == 1)
	{
		/// - The increment does the counter zero interrupt count.
		t16bCh0IntZero++;
	   /**
		* @todo add interrupt function of T16B counter zero interrupt
		*/
	}

	/// Check counter MAX interrupt flag.
	if(t16b0intf_set.bit.CNTMAXIF == 1)
	{
		/// - The increment does the counter max interrupt count.
		t16bCh0IntMax++;

	   /**
		* @todo add interrupt function of T16B counter max interrupt
		*/
		/// Set trigger signal(LOW).
		{
			t16b0ccctl1_set.reg = T16B0CCCTL1;
			t16b0ccctl1_set.bit.CAPIS = 2;
			T16B0CCCTL1 = t16b0ccctl1_set.reg;
		}
	}

	/// Check compare/capture 0 interrupt flag.
	if(t16b0intf_set.bit.CMPCAP0IF == 1)
	{
		/// - The increment does the comparator interrupt count.
		t16bCh0IntComparator++;

	   /**
		* @todo add interrupt function of T16B compare/capture 0  interrupt
		*/
		/// Set trigger signal(HIGH).
		{
			t16b0ccctl1_set.reg = T16B0CCCTL1;
			t16b0ccctl1_set.bit.CAPIS = 3;
			T16B0CCCTL1 = t16b0ccctl1_set.reg;
		}
	}

	if(t16b0intf_set.bit.CAPOW0IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 0 over write  interrupt
		*/
	}

	/// Check compare/capture 1 interrupt flag.
	if(t16b0intf_set.bit.CMPCAP1IF == 1)
	{
		/// - The increment does the compare interrupt count.
		t16bCh0IntCapture++;
		/**
		* @todo add interrupt function of T16B compare/capture 1  interrupt
		*/
		/// - Get timer counter data.
		if (t16bCh0IntCaptureIndex < T16B_CH0_INT_CAP_DATA_MAX) {
			t16bCh0IntCaptureData[ t16bCh0IntCaptureIndex ] = T16B0CCR1;
			t16bCh0IntCaptureIndex++;
		}
	}

	if(t16b0intf_set.bit.CAPOW1IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 1 over write  interrupt
		*/
	}

	if(t16b0intf_set.bit.CMPCAP2IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 2 interrupt
		*/

	}
	if(t16b0intf_set.bit.CAPOW2IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 2 over write  interrupt
		*/
	}
	if(t16b0intf_set.bit.CMPCAP3IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 3 interrupt
		*/

	}
	if(t16b0intf_set.bit.CAPOW3IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 3 over write  interrupt
		*/
	}
	if(t16b0intf_set.bit.CMPCAP4IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 4 interrupt
		*/

	}
	if(t16b0intf_set.bit.CAPOW4IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 4 over write  interrupt
		*/
	}
	if(t16b0intf_set.bit.CMPCAP5IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 5 interrupt
		*/

	}
	if(t16b0intf_set.bit.CAPOW5IF == 1)
	{
		/**
		* @todo add interrupt function of T16B capture 5 over write  interrupt
		*/
	}
}

#endif

